#include "common.h"

/* *** ANSI code returning length of a file *** */

int file_length(FILE *fptr) {
  int fileSize;
  int posit;

  posit=ftell(fptr);
  fseek(fptr,0,SEEK_END);
  fileSize = ftell(fptr);
  fseek(fptr,posit,SEEK_SET);

  return fileSize;
}

/* *** Returns a pointer to the first match of s in ch *** */

char *search_block(char *ch, char *s, int chLen, int sLen) {
  int i,j;
  int nomatch;

  if (sLen > chLen) {
    return NULL;
  }

  for (i=0;i<=chLen-sLen;i++) {
    nomatch=0;
    for (j=0;j<sLen && nomatch==0;j++) {
      if (ch[i+j] != s[j]) {
	nomatch=1;
      }
    }
    if (nomatch == 0) {
      return &ch[i];
    }
  }
  
  return NULL;
}

/* *** If possible, updates file.bin in a CD image file ** */

int file_patch(char *fileOrigin, char *fileModif, char *isoImage, int fileID) {
  FILE *fconfig;
  FILE *fOrg;
  FILE *isoFile;
  FILE *fNew;
  int fileOffset[2] = {-1,-1};
  int fileSize;
  int newFileSize;
  int isoSize;
  int blockSize;
  char *configFile;
  char *buffer;
  char *buffertemp;
  char *bufferptr;
  char *newKBuffer;

  /* Opening isoImage */
  isoFile = fopen(isoImage,"rb+");
  if (isoFile == NULL) {
    printf("file_patch : ");
    printf(ERROR_OPEN_FILE,isoImage);
    return 2;
  }
  isoSize = file_length(isoFile);

  /* Looking for file.bin size */
  fOrg = fopen(fileOrigin,"rb");
  if (fOrg == NULL) {
    printf("file_patch : ");
    printf(ERROR_OPEN_FILE,fileOrigin);
    return 1;
  }
  fileSize = file_length(fOrg);
  fclose(fOrg);

  /* Looking for a configuration file */
  configFile = (char *) malloc((5+strlen(isoImage))*sizeof(char));
  sprintf(configFile,"%s.cfg",isoImage);
  fconfig = fopen(configFile,"rb");

  if (fconfig != NULL) {
    /* If a configuration file exists, we don t have to look for file.bin offset */
    fread(&fileOffset,sizeof(int),ID_COUNT,fconfig);
    fclose(fconfig);
  }

  if (fileOffset[fileID] == -1) {
    /* Looking for file.bin in file isoImage */
    fOrg = fopen(fileOrigin,"rb");
    if (fOrg == NULL) {
      printf("file_patch : ");
      printf(ERROR_OPEN_FILE,fileOrigin);
      return 1;
    }
    fileSize = file_length(fOrg);
    buffer = (char *) malloc(fileSize*sizeof(char));
    fread(buffer,sizeof(char),fileSize,fOrg);
    fclose(fOrg);
    buffertemp = (char *) malloc((fileSize*10)*sizeof(char));
    while (ftell(isoFile) < isoSize) {
      if (ftell(isoFile) + fileSize * 10 <= isoSize) {
	blockSize = fileSize * 10;
      } else {
	blockSize = isoSize - ftell(isoFile);
      }
      fread(buffertemp,sizeof(char),blockSize,isoFile);
      bufferptr = search_block(buffertemp,buffer,blockSize,fileSize);
      if (bufferptr != NULL) {
	fileOffset[fileID] = ftell(isoFile) - blockSize + ((int) (bufferptr - buffertemp));
	fseek(isoFile,0,SEEK_END);
      }
    }
    free(buffertemp);
    free(buffer);

    if (fileOffset[fileID] != -1) {
      /* Saving offset in a config file */
      fconfig = fopen(configFile,"wb");
      if (fconfig != NULL) {
	fwrite(&fileOffset,sizeof(int),ID_COUNT,fconfig);
	fclose(fconfig);
      } else {
	printf("file_patch : ");
	printf(ERROR_CREA_FILE,configFile);
      }
    }
  }

  if (fileOffset[fileID] == -1) {
    free(configFile);
    fclose(isoFile);
    printf("file_patch : ");
    printf(ERROR_FIND_FILE_IN,fileOrigin,isoImage);
    return 3;
  }

  /* Opening modified file file */
  fNew = fopen(fileModif,"rb");
  if (fNew == NULL) {
    printf("file_patch : ");
    printf(ERROR_OPEN_FILE,fileModif);
    return 4;
  }
  newFileSize = file_length(fNew);
  newKBuffer = (char *) malloc(newFileSize*sizeof(char));
  fread(newKBuffer,sizeof(char),newFileSize,fNew);
  fclose(fNew);
  if (newFileSize != fileSize) {
    printf("file_patch : ");
    if (newFileSize > fileSize) {
      printf(FILE_BIGGER,fileModif,newFileSize,fileOrigin,fileSize);
    } else {
      printf(FILE_SMALLER,fileModif,newFileSize,fileOrigin,fileSize);
    }
  }


  /* Writing modified file into isoFile */
  fseek(isoFile,fileOffset[fileID],SEEK_SET);
  fwrite(newKBuffer,sizeof(char),newFileSize,isoFile);
  fclose(isoFile);
  free(newKBuffer);
 
  return 0;
}
