/*
  FF7 kernel.bin file routines
  contact : Fremen@planet-shitfliez.org   
*/

#include "kernelbin.h"

/* *** loads a kernel.bin file *** */

kernel_t *kernel_load(char *fileName) {
  kernel_t *kernel;
  FILE *fptr;
  unsigned short int blockDesc[3];
  int i;
  unsigned long int blockSize;
  char *buffer;
  FILE *tempFile;
  gzFile *gzip;

  fptr = fopen(fileName,"rb");
  if (fptr == NULL) {
    printf("kernel_load : ");
    printf(ERROR_OPEN_FILE,fileName);
    return NULL;
  }

  /* Creation of a new kernel var */
  kernel = (kernel_t *) malloc(sizeof(kernel_t));

  /* Number of blocks count */
  kernel->nbBlocks = 0;
  do {
    fread(&blockDesc[0],sizeof(unsigned short int),1,fptr);
    if (blockDesc[0] != 0) {
      kernel->nbBlocks++;
      fseek(fptr,(int) blockDesc[0] + 4,SEEK_CUR); /* Moving to the next block */
    }
  } while (blockDesc[0] != 0);
  fseek(fptr,0,SEEK_SET); /* We return to the file beginning */
  
  /* Allocation of kernel structure tables */
  kernel->blockNum  = (unsigned short int *) malloc(kernel->nbBlocks*sizeof(unsigned short int));
  kernel->blockSize = (unsigned short int *) malloc(kernel->nbBlocks*sizeof(unsigned short int));
  kernel->blockData = (char **) malloc(kernel->nbBlocks*sizeof(char *));

  /* Blocks extraction */
  for (i=0;i<kernel->nbBlocks;i++) {

    /* Block header */
    fread(&blockDesc[0],sizeof(unsigned short int),3,fptr);
    kernel->blockNum[i] = blockDesc[2];
    kernel->blockSize[i] = blockDesc[1];

    /* Compressed data block */
    buffer = (char *) malloc((blockDesc[0])*sizeof(char));
    fread(buffer,sizeof(char),blockDesc[0],fptr);

    /* Data uncompression, using a temporary file */
    kernel->blockData[i] = (char *) malloc(blockDesc[1]*sizeof(char));
    blockSize = (unsigned long int) blockDesc[1];
    tempFile = fopen("~temp.dat","wb");
    if (tempFile == NULL) {
      printf("kernel_load : ");
      printf(ERROR_CREA_TEMPD_FILE);
      return NULL;
    }
    fwrite(buffer,sizeof(char),blockDesc[0],tempFile);
    fclose(tempFile);

    gzip = gzopen("~temp.dat","rb");
    if (gzip == NULL) {
      printf("kernel_load : ");
      printf(ERROR_OPEN_TEMPD_FILE);
      return NULL;
    }
    gzread(gzip,kernel->blockData[i],blockSize);
    gzclose(gzip);
    remove("~temp.dat");

    free(buffer);
  }

  fclose(fptr);  

  return kernel;
}

/* *** saves a kernel.bin file *** */

int kernel_save(char *fileName, kernel_t *kernel) {
  FILE *fptr;
  int i;
  unsigned short int blockDesc[3];
  char *buffer;
  FILE *tempFile;
  gzFile *gzip;
  int fileSize;

  /* Creating asked file */
  fptr = fopen(fileName,"rb");
  if (fptr != NULL) {
    printf("kernel_save : ");
    printf(FILE_OVERWRITE,fileName);
    fclose(fptr);
  }

  fptr = fopen(fileName,"wb");
  if (fptr == NULL) {
    printf("kernel_save : ");
    printf(ERROR_CREA_FILE,fileName);
    return 1;
  }
  
  for (i=0;i<kernel->nbBlocks;i++) {
    blockDesc[1] = kernel->blockSize[i];
    blockDesc[2] = kernel->blockNum[i];

    /* Data compression, using a temporary file */
    gzip = gzopen("~temp.dat","wb");
    if (gzip == NULL) {
      printf("kernel_save : ");
      printf(ERROR_CREA_TEMPD_FILE);
      return 2;
    }
    if (gzsetparams (gzip, 9, Z_DEFAULT_STRATEGY) != Z_OK) {
      printf("kernel_save : ");
      printf(GZ_SET_PARAMS);
    }
    gzwrite(gzip,kernel->blockData[i],kernel->blockSize[i]);
    gzclose(gzip);

    tempFile = fopen("~temp.dat","rb");
    fileSize = file_length(tempFile);
    buffer = (char *) malloc(fileSize*sizeof(char));
    fread(buffer,sizeof(char),fileSize,tempFile);
    fclose(tempFile);
    remove("~temp.dat");

    /* Writing block in fptr */
    blockDesc[0] = (unsigned short int) fileSize;
    fwrite(&blockDesc[0],sizeof(unsigned short int),3,fptr);
    fwrite(buffer,sizeof(char),fileSize,fptr);

    free(buffer);
  }

  /* Writing end of file */
  blockDesc[0] = 0;
  fwrite(&blockDesc[0],sizeof(unsigned short int),1,fptr);
  fclose(fptr);

  return 0;
}

/* *** clears a kernel_t structure *** */

void clear_kernel(kernel_t *kernel) {
  int i;

  if (kernel != NULL) {
    for (i=0;i<kernel->nbBlocks;i++) {
      if (kernel->blockData[i] != NULL) {
	free(kernel->blockData[i]);
      }
    }
    if (kernel->blockData != NULL) {
      free(kernel->blockData);
    }
    if (kernel->blockNum != NULL) {
      free(kernel->blockNum);
    }
    if (kernel->blockSize != NULL) {
      free(kernel->blockSize);
    }
  }

  return;
}
