#include "cdformat.h"

/* *** Converts a PSX .bin image file to .iso *** */

int bin2iso(char *binFile, char *isoFile) {
  FILE *source;
  FILE *target;
  char buffer[2352];
  int fileSize;
  FILE *headFile;
  char *headFileName;

  /* Opening .bin file */
  source = fopen(binFile,"rb");
  if (source == NULL) {
    printf("bin2iso : ");
    printf(ERROR_OPEN_FILE,binFile);
    return 1;
  }
  fileSize = file_length(source);

  /* Creating a .h to store headers and subheaders data */
  headFileName = (char *) malloc((strlen(binFile)+3)*sizeof(char));
  sprintf(headFileName,"%s.h",binFile);
  headFile = fopen(headFileName,"wb");
  if (headFile == NULL) {
    printf("bin2iso : ");
    printf(ERROR_CREA_FILE,headFileName);
    return 3;
  }

  /* ISO file detection */
  target = fopen(isoFile,"rb");
  if (target != NULL) {
    fclose(target);
    printf("bin2iso : ");
    printf(FILE_OVERWRITE,isoFile);
  }

  /* ISO file creation */
  target = fopen(isoFile,"wb");
  if (target == NULL) {
    printf("bin2iso : ");
    printf(ERROR_CREA_FILE,isoFile);
    return 2;
  }

  /* Convertion loop */
  while (ftell(source) < fileSize) {
    fread(buffer,sizeof(char),2352,source);
    fwrite(&buffer[24],sizeof(char),2048,target); /* Sync-Header-SubHeader-EDC-ECC truncation */
    fwrite(&buffer[12],sizeof(char),12,headFile); /* Header/Subheader backup */
  }
  fclose(target);
  fclose(headFile);
  fclose(source);

  free(headFileName);

  return 0;
}

/* *** Converts a PSX .nrg image file to .iso *** */

int nrg2iso(char *nrgFile, char *isoFile) {
  FILE *source;
  FILE *target;
  char buffer[2352];
  int fileSize;
  FILE *headFile;
  char *headFileName;

  /* Opening .nrg file */
  source = fopen(nrgFile,"rb");
  if (source == NULL) {
    printf("nrg2iso : ");
    printf(ERROR_OPEN_FILE,nrgFile);
    return 1;
  }
  fileSize = file_length(source);
  fseek(source,307200,SEEK_SET); /* Skipping 300 Kbytes Nero header */

  /* Creating a .h to store headers and subheaders data */
  headFileName = (char *) malloc((strlen(nrgFile)+3)*sizeof(char));
  sprintf(headFileName,"%s.h",nrgFile);
  headFile = fopen(headFileName,"wb");
  if (headFile == NULL) {
    printf("nrg2iso : ");
    printf(ERROR_CREA_FILE,headFileName);
    return 3;
  }

  /* ISO file detection */
  target = fopen(isoFile,"rb");
  if (target != NULL) {
    fclose(target);
    printf("nrg2iso : ");
    printf(FILE_OVERWRITE,isoFile);
  }

  /* ISO file creation */
  target = fopen(isoFile,"wb");
  if (target == NULL) {
    printf("nrg2iso : ");
    printf(ERROR_CREA_FILE,isoFile);
    return 2;
  }

  /* Convertion loop */
  while (ftell(source) < fileSize) {
    fread(buffer,sizeof(char),2352,source);
    fwrite(&buffer[24],sizeof(char),2048,target); /* Sync-Header-SubHeader-EDC-ECC truncation */
    fwrite(&buffer[12],sizeof(char),12,headFile); /* Header/Subheader backup */
  }
  fclose(target);
  fclose(headFile);
  fclose(source);

  free(headFileName);

  return 0;
}

/* *** Converts a .iso file to a PSX .bin image file *** */

int iso2bin(char *isoFile, char *binFile) {
  FILE *source;
  FILE *target;
  char buffer[2352];
  int fileSize;
  FILE *headFile;
  char *headFileName;
  char header[4];

  /* Opening .iso file */
  source = fopen(isoFile,"rb");
  if (source == NULL) {
    printf("iso2bin : ");
    printf(ERROR_OPEN_FILE,isoFile);
    return 1;
  }
  fileSize = file_length(source);

  /* Reading .h file (where is stored headers and subheaders data) */
  headFileName = (char *) malloc((strlen(binFile)+3)*sizeof(char));
  sprintf(headFileName,"%s.h",binFile);
  headFile = fopen(headFileName,"rb");
  if (headFile == NULL) {
    printf("iso2bin : ");
    printf(ERROR_OPEN_FILE,headFileName);
    return 3;
  }

  /* .bin image detection */
  target = fopen(binFile,"rb");
  if (target != NULL) {
    fclose(target);
    printf("iso2bin : ");
    printf(FILE_OVERWRITE,binFile);
  }

  /* Creating .bin file */
  target = fopen(binFile,"wb");
  if (target == NULL) {
    printf("iso2bin : ");
    printf(ERROR_CREA_FILE,binFile);
    return 2;
  }

  /* Convertion loop */
  while (ftell(source) < fileSize) {
    fread(&buffer[24],sizeof(char),2048,source);
    fread(&header[0] ,sizeof(char),4,headFile);
    fread(&buffer[16],sizeof(char),8,headFile);
    do_encode_L2((unsigned char *) buffer,MODE_2_FORM_1,150);
    memcpy(&buffer[12],&header[0],4*sizeof(char));
    fwrite(buffer,sizeof(char),2352,target);
  }
  
  fclose(target);
  fclose(headFile);
  fclose(source);

  free(headFileName);

  return 0;
}
