#include "scene.h"

/* *** Loads a scene.bin file into a scene structure *** */

scene_t *scene_load(char *sceneFileName) {
  FILE *sceneFile;
  FILE *outFile;
  char outFileName[256];
  int sceneFileSize;
  int filenum=0;
  int i,dec,endloop;
  unsigned char *buffer, *bufferExp;
  unsigned int *pointers; /* Each 0x2000 bytes block begins with 16 pointers
                             Position_of_data_in_bytes = Offset_block + pointer * 4; */
  scene_t *scene;
  gzFile *gzip;

  /* Looking for scene.bin file */
  sceneFile = fopen(sceneFileName, "rb");
  if (sceneFile == NULL) {
    printf("scene_load : ");
    printf(ERROR_OPEN_FILE, sceneFileName);
    return NULL;
  }

  /* Loading scene.bin file into memory */
  sceneFileSize = file_length(sceneFile);

  /* Allocating scene structure */
  scene = (scene_t *) malloc(sizeof(scene_t));
  
  /* Reads each archived file and writes them in individual files */
  buffer = (unsigned char *) malloc(sizeof(unsigned char) * 0x2000);
  bufferExp = (unsigned char *) malloc(sizeof(unsigned char) * 7808);
  do {
    fread(buffer,sizeof(char),0x2000,sceneFile);
    pointers=(unsigned int *) buffer;
    
    i=0;
    endloop=0;
    while (endloop == 0) {

      /* Creating file */
      sprintf(outFileName,"./~file%d.gz",filenum);
      outFile = fopen(outFileName,"wb");
      if (outFile == NULL) {
	printf("scene_load : Unable to create temporary file %s\n",outFileName);
	return NULL;
      }

      /* Last pointer must point end of block adress */
      if (pointers[i+1] == 0xFFFFFFFF) {
	pointers[i+1] = 0x800;
	endloop=1;
      }

      /* Sometimes 0xFF are added in block end */
      dec=4*pointers[i+1]-1;
      if (buffer[dec] == 0xFF) {
	while (buffer[dec] == 0xFF) {
	  dec--;
	}
      }
      dec++;

      /* Writing block in temporary file */
      fwrite(&buffer[pointers[i]*4],sizeof(unsigned char),dec-4*pointers[i],outFile);
      fclose (outFile);

      /* Now we uncompress the file, using zlib */
      gzip = gzopen(outFileName,"rb");
      if (gzip == NULL) {
	printf("scene_load : ");
	printf(ERROR_OPEN_TEMP_FILE,outFileName);
	return NULL;
      }
      gzread(gzip,bufferExp,7808);
      gzclose(gzip);
      remove(outFileName);
      
      /* Writing the uncompressed file */
      outFile = fopen("./~temp.dat","wb");
      if (outFile == NULL) {
	printf("scene_load : ");
	printf(ERROR_CREA_TEMPD_FILE);
	return NULL;
      }
      fwrite(bufferExp,sizeof(unsigned char),7808,outFile);
      fclose(outFile);

      /* Loading data in scene structure */
      scene->file[filenum] = file_load("./~temp.dat");
      remove("./~temp.dat");   

      filenum++;
      i++;
    }

  } while (ftell(sceneFile) < sceneFileSize);

  fclose (sceneFile);
  
  free(buffer);
  free(bufferExp);

  return scene;
}

/* *** Loads an uncompress "file" (i.e. a four scenes block) *** */

file_type_t *file_load(char *fileName) {
  file_type_t *file;

  FILE *inFile;
  int i,j,k,l,n;
  unsigned char *buffer;
  int subPointers[4][17]; /* 0 : relative offset of a block (including subpointers) */
                          /* 1-16 : pointer to the nth subblock */
  
  /* Looking for file */
  inFile = fopen(fileName,"rb");
  if (inFile == NULL) {
    printf("file_load : ");
    printf(ERROR_OPEN_FILE,fileName);
    return NULL;
  }
  
  /* File structure allocation */
  file = (file_type_t *) malloc(sizeof(file_type_t));

  /* Initializing all scripts */
  for (i=0;i<3;i++) {
    for (j=0;j<16;j++) {
      file->scripts.block[i].subBlock[j] = NULL;
      file->scripts.block[i].subBlockSize[j] = 0;
      file->scripts.block[i].subPointer[j] = -1;
    }
  }

  /* Reading constant length data */
  fread(file->monsterID,sizeof(unsigned short int),4,inFile);
  fread(file->battle,sizeof(battle_scene_t),4,inFile);
  fread(file->battle_cam,sizeof(battle_scene_camera_t),4,inFile);
  for (i=0;i<4;i++) {
    fread(file->battle_spec[i],sizeof(battle_scene_spec_t),6,inFile);
  }
  fread(file->monsterSpec,sizeof(monster_carac_t),3,inFile);
  fread((void *) &(file->attacks),sizeof(attacks_names_t),1,inFile);
  


  /* *************** Reading variable length data (scripts) *************** */

  fread(file->scripts.pointer,sizeof(short int),3,inFile);

  /* Reading relative offset of each block (including subpointers) */
  /* and grouping valid (not -1) pointers                          */
  n=0;
  for (j=0;j<3;j++) {
    if (file->scripts.pointer[j] != -1) {
      subPointers[n][0] = file->scripts.pointer[j];
      n++;
    }
  }

  /* Seeking at the beginning of the file script part */
  fseek(inFile,0xE80,SEEK_SET);

  /* Looking for the end of the last script block */
  k=ftell(inFile);
  buffer = (unsigned char *) malloc(sizeof(unsigned char)*(7808-k));
  fread(buffer,sizeof(unsigned char),7808-k,inFile);
  fseek(inFile,k+6,SEEK_SET);
  k=7808-k-1;
  while (buffer[k] == 0xFF && k>0) { /* Skipping ending 0xFF bytes */
    k--;
  }
  while (buffer[k] != 0x73 && k>0) { /* Looking for a 0x73 'end of script' byte */
    k--;
  }
  subPointers[n][0] = k+1; /* End of last script block at offset k+1 (relative to file offset 0xE80) */
  free(buffer);


  l=0;
  for (i=0;i<3;i++) {
    if (file->scripts.pointer[i] != -1) {

      /* If pointer number i exists : seeking to the pointed offset */
      fseek(inFile,0x0E80+(unsigned int) file->scripts.pointer[i],SEEK_SET);

      /* Reading the subpointers block (i.e. pointers to script into a block) */
      fread(file->scripts.block[i].subPointer,sizeof(short int),16,inFile);
      
      /* Grouping valid (not -1) subpointers */
      n=1;
      for (j=0;j<16;j++) {
	subPointers[l][j+1]=-1;
      }
      for (j=0;j<16;j++) {
	if (file->scripts.block[i].subPointer[j] != -1) {
	  subPointers[l][n]=file->scripts.block[i].subPointer[j];
	  n++;
	}
      }

      /* Reading scripts */
      n=1;
      for (j=0;j<16;j++) {
	if (file->scripts.block[i].subPointer[j] != -1) {
	  
	  /* Finding script size */
	  if (subPointers[l][n+1] != -1) {
	    /* If there is another script in the same block, size is (next_script_offset - script-offset) */
	    file->scripts.block[i].subBlockSize[j]=subPointers[l][n+1]-subPointers[l][n];
	  } else {
	    /* If we are on the last script, size is (next_block - script_offset) */
	    file->scripts.block[i].subBlockSize[j]=(subPointers[l+1][0]) - (subPointers[l][0]+subPointers[l][n]);
	  }
	  n++;

	  /* Allocating and reading script */
	  file->scripts.block[i].subBlock[j] = (unsigned char *) malloc(sizeof(unsigned char)*file->scripts.block[i].subBlockSize[j]);
	  fread(file->scripts.block[i].subBlock[j],sizeof(unsigned char),file->scripts.block[i].subBlockSize[j],inFile);

	  /* "Deleting" the 0xFF there should be at the end of subBlock */
	  if (file->scripts.block[i].subBlockSize[j] > 0) {
	    if (file->scripts.block[i].subBlock[j][file->scripts.block[i].subBlockSize[j]-1] == 0xFF) {
	      file->scripts.block[i].subBlockSize[j]--;
	    }
	  }
	  
	} else { /* If there is no script */
	  file->scripts.block[i].subBlock[j] = NULL;
	  file->scripts.block[i].subBlockSize[j] = 0;
	}
      } /* End of block loop */
      
      l++;
      
    }
  } /* End of script loop */
  
  /* ********************************************************************** */

  fclose (inFile);

  return file;
}

/* *** Erases a scene structure from memory *** */

void scene_clear(scene_t *scene) {
  int filenum,i,j;
  
  if (scene == NULL) {
    return;
  }

  for (filenum=0;filenum<256;filenum++) {
    if (scene->file[filenum] != NULL) {
      for (i=0;i<3;i++) {
	if (scene->file[filenum]->scripts.pointer[i] != -1) {
	  for (j=0;j<16;j++) {
	    if (scene->file[filenum]->scripts.block[i].subPointer[j] != -1) {
	      if (scene->file[filenum]->scripts.block[i].subBlock[j] != NULL) {
		free(scene->file[filenum]->scripts.block[i].subBlock[j]);
	      }
	    }
	  }
	}
      }
      free(scene->file[filenum]);
    }
  }
  free(scene);

  return;
}

/* *** Saves an uncompress "file" (i.e. a four scenes block) *** */

void file_save(char *fileName, file_type_t *file) {

  FILE *outFile;
  int i,j;
  unsigned char *buffer;
  short int offset,subOffset;
  int existsScript;
  
  /* Looking for file */
  outFile = fopen(fileName,"wb");
  if (outFile == NULL) {
    printf("file_save : ");
    printf(ERROR_CREA_FILE,fileName);
    return;
  }
  
  /* Writing constant length data */
  fwrite(file->monsterID,sizeof(unsigned short int),4,outFile);
  fwrite(file->battle,sizeof(battle_scene_t),4,outFile);
  fwrite(file->battle_cam,sizeof(battle_scene_camera_t),4,outFile);
  for (i=0;i<4;i++) {
    fwrite(file->battle_spec[i],sizeof(battle_scene_spec_t),6,outFile);
  }
  fwrite(file->monsterSpec,sizeof(monster_carac_t),3,outFile);
  fwrite((void *) &(file->attacks),sizeof(attacks_names_t),1,outFile);
  


  /* *************** Writing variable length data (scripts) *************** */

  /* Writing FF bytes */
  buffer = (unsigned char *) malloc((7808 - 0xE80)*sizeof(unsigned char));
  for (i=0;i<7808-0xE80;i++) {
    buffer[i] = 0xFF;
  }
  fwrite(buffer,sizeof(unsigned char),7808-0xE80,outFile);
  fseek(outFile,0xE80,SEEK_SET);
  free(buffer);

  /* Updating pointers/subpointers values */
  offset=6;
  for (i=0;i<3;i++) {

    /* Setting pointer i value */
    existsScript=0;
    for (j=0;j<16 && existsScript==0;j++) {
      if (file->scripts.block[i].subBlockSize[j] != 0) {
	existsScript=1;
      }
    }

    /* Skipping pointer if there is no script */
    if (existsScript == 0) {
      file->scripts.pointer[i] = -1;

    /* Else setting pointer value */
    } else {
      file->scripts.pointer[i] = offset;
      subOffset = 32;
      offset = offset + 32;

      /* Setting subpointer values */
      for (j=0;j<16;j++) {
	if (file->scripts.block[i].subBlockSize[j] != 0) {
	  file->scripts.block[i].subPointer[j] = subOffset;
	  subOffset = subOffset + file->scripts.block[i].subBlockSize[j];
	  offset    = offset    + file->scripts.block[i].subBlockSize[j];
	} else {
	  file->scripts.block[i].subPointer[j] = -1;
	}
      }

      /* Offset must be even */
      if (offset % 2 == 1) {
	offset++;
      }
    }
   
  } /* End of for (i=0;i<3;i++) */

  fwrite(file->scripts.pointer,sizeof(short int),3,outFile);

  for (i=0;i<3;i++) {

    /* Writing scripts data */
    if (file->scripts.pointer[i] != -1) {
      fwrite(file->scripts.block[i].subPointer,sizeof(short int),16,outFile);
      for (j=0;j<16;j++) {
	if (file->scripts.block[i].subBlockSize[j] > 0) {
	  fwrite(file->scripts.block[i].subBlock[j],sizeof(unsigned char),
		 file->scripts.block[i].subBlockSize[j],outFile);
	}
      }
    }

    /* Offset of next script block must be even */
    if (ftell(outFile) % 2 == 1) {
      fseek(outFile,1,SEEK_CUR);
    }

  }
  
  /* ********************************************************************** */

  fclose (outFile);

  return;
}

/* *** Saves a scene structure in the specified file *** */

int scene_save(char *sceneFileName, scene_t *scene, unsigned char *blockTable) {
  FILE *fptr;
  FILE *sceneFile;
  unsigned char *buffer;
  int filenum;
  int blockFileCount;
  int currentBlockSize;
  int i;
  int offset;
  int fileSize;
  int blockCount=0;
  gzFile *gzip;
  unsigned char *buffertemp;

  /* Verification of scene validity */
  if (scene == NULL) {
    printf("scene_save : ");
    printf(SCENE_NULL);
    return 1;
  }
  for (i=0;i<256;i++) {
    if (scene->file[i] == NULL) {
      printf("scene_save : ");
      printf(SCENE_FILE_NULL,i);
      return 2;
    }
  }

  /* Creating asked file */
  sceneFile = fopen(sceneFileName,"rb");
  if (sceneFile != NULL) {
    printf("scene_save : ");
    printf(FILE_OVERWRITE,sceneFileName);
    fclose(sceneFile);
  }
  sceneFile = fopen(sceneFileName,"wb");
  if (sceneFile == NULL) {
    printf("scene_save : ");
    printf(ERROR_CREA_FILE,sceneFileName);
    return 5;
  }

  /* Allocating buffer */
  buffer = (unsigned char *) malloc(0x2000*sizeof(unsigned char));

  /* Initializing block data */
  for (i=0;i<0x2000;i++) {
    buffer[i] = 0xFF;
  }
  currentBlockSize=16*sizeof(int); /* Pointers are included in a 0x2000 bytes block */
  offset=16;
  blockTable[blockCount++] = 0x00;
  blockFileCount=0;

  /* Beginning of loop */
  for (filenum=0;filenum<256;filenum++) {
   
    /* Writing file to a temporary file */
    file_save("./~temp.dat",scene->file[filenum]);

    /* Loading the file */
    fptr = fopen("./~temp.dat","rb");
    if (fptr == NULL) {
      printf("scene_save : ");
      printf(ERROR_OPEN_TEMPD_FILE);
      return 4;
    }
    fileSize = file_length(fptr);
    buffertemp = (unsigned char *) malloc(fileSize*sizeof(unsigned char));
    fread(buffertemp,sizeof(unsigned char),fileSize,fptr);
    fclose(fptr);
    remove("./~temp.dat");

    /* Compressing the file, using zlib */
    gzip = gzopen("./~temp.dat","wb");
    if (gzip == NULL) {
      printf("scene_save : ");
      printf(ERROR_CREA_TEMPD_FILE);
      return 3;
    }
    if (gzsetparams (gzip, 9, Z_DEFAULT_STRATEGY) != Z_OK) {
      printf("scene_save : ");
      printf(GZ_SET_PARAMS);
    }
    gzwrite(gzip,buffertemp,7808);
    gzclose(gzip);
    free(buffertemp);
    
    /* Loading compressed file */
    fptr = fopen("./~temp.dat","rb");
    if (fptr == NULL) {
      printf("scene_save : ");
      printf(ERROR_OPEN_TEMPD_FILE);
      return 4;
    }
    fileSize = file_length(fptr);
    buffertemp = (unsigned char *) malloc(fileSize*sizeof(unsigned char));
    fread(buffertemp,sizeof(unsigned char),fileSize,fptr);
    fclose(fptr);
    remove("./~temp.dat");

    /* Creation of a new block, if needed */
    if (fileSize+currentBlockSize > 0x2000) {
      fwrite(buffer,sizeof(unsigned char),0x2000,sceneFile);
      blockTable[blockCount++] = (unsigned char) filenum;
      
      /* Initializing new block data */
      for (i=0;i<0x2000;i++) {
	buffer[i] = 0xFF;
      }
      currentBlockSize=16*sizeof(int); /* Pointers are included in a 0x2000 bytes block */
      offset=16;
      blockFileCount=0;
    }

    /* Adding compressed file in current block */
    memcpy(&buffer[offset*4],buffertemp,fileSize);
    *((int *) (&buffer[blockFileCount*4])) = offset;
    offset+=fileSize/4;
    if (fileSize % 4 != 0) {
      offset++;
    }
    currentBlockSize = offset * 4;
    blockFileCount++;

  } /* End of loop */

  /* Saving remaining data, if any */
  fwrite(buffer,sizeof(unsigned char),0x2000,sceneFile);
      
  /* Freeing block data */
  free(buffer);
  
  /* Initializing remaining table elements */
  while (blockCount<64) {
    blockTable[blockCount++] = 0xFF;
  }

  fclose(sceneFile);

  return 0;
}

/* *** Exports a whole scene structure to a text file *** */

int scene_export_text(scene_t *scene, char *fileName) {
  FILE *fptr;
  int i,j,k,cr;
  int filenum;
  char *buffer;

  /* Verification of scene validity */
  if (scene == NULL) {
    printf("scene_export_text : ");
    printf(SCENE_NULL);
    return 1;
  }
  for (i=0;i<256;i++) {
    if (scene->file[i] == NULL) {
      printf("scene_export_text : ");
      printf(SCENE_FILE_NULL,i);
      return 2;
    }
  }
  
  /* Trying to create output file */
  fptr = fopen(fileName,"rb");
  if (fptr != NULL) {
    printf("scene_export_text : ");
    printf(FILE_OVERWRITE,fileName);
    fclose(fptr);
  }
  fptr = fopen(fileName,"wb");
  if (fptr == NULL) {
    printf("scene_export_text : ");
    printf(ERROR_CREA_FILE,fileName);
    return 3;
  }

  fprintf(fptr,EXPORT_HEADER);
  fprintf(fptr,EXPORT_WARNING);

  /* Main loop */
  for (filenum=0;filenum<256;filenum++) {
    fprintf(fptr,"\n\n\n");
    fprintf(fptr,"*-------------------------------------------------------------------------------*\n");
    fprintf(fptr,"| ############################### FILE %03d/255 ################################ |\n",
	    filenum);
    fprintf(fptr,"*---------------*---------------*---------------*---------------*---------------*\n");

    /* Monster ID */
    fprintf(fptr,"| MonsterID     | [0x%04x]      | [0x%04x]      | [0x%04x]      | [0x%04x]      |\n",
	    scene->file[filenum]->monsterID[0],scene->file[filenum]->monsterID[1],
	    scene->file[filenum]->monsterID[2],scene->file[filenum]->monsterID[3]);
    
    /* Battle global information */
    fprintf(fptr,"*---------------*---------------*---------------*---------------*---------------*\n");
    fprintf(fptr,"| ========================= BATTLE GLOBAL INFORMATION ========================= |\n");
    fprintf(fptr,"*---------------*---------------*---------------*---------------*---------------*\n");
    fprintf(fptr,"|     TYPE      |   BATTLE 1    |   BATTLE 2    |   BATTLE 3    |   BATTLE 4    |\n");
    fprintf(fptr,"*---------------*---------------*---------------*---------------*---------------*\n");
    fprintf(fptr,"|    placeID    | [0x%04x]      | [0x%04x]      | [0x%04x]      | [0x%04x]      |\n",
	    (unsigned short int) scene->file[filenum]->battle[0].placeID,
	    (unsigned short int) scene->file[filenum]->battle[1].placeID,
	    (unsigned short int) scene->file[filenum]->battle[2].placeID,
	    (unsigned short int) scene->file[filenum]->battle[3].placeID);
    for (i=0;i<3;i++) {
      fprintf(fptr,"| unknown val.1 | [0x%04x]      | [0x%04x]      | [0x%04x]      | [0x%04x]      |\n",
	      (unsigned short int) scene->file[filenum]->battle[0].unknown[i],
	      (unsigned short int) scene->file[filenum]->battle[1].unknown[i],
	      (unsigned short int) scene->file[filenum]->battle[2].unknown[i],
	      (unsigned short int) scene->file[filenum]->battle[3].unknown[i]);
    }
    for (i=0;i<4;i++) {
      fprintf(fptr,"| unknown val.2 | [0x%04x]      | [0x%04x]      | [0x%04x]      | [0x%04x]      |\n",
	      (unsigned short int) scene->file[filenum]->battle[0].unknown2[i],
	      (unsigned short int) scene->file[filenum]->battle[1].unknown2[i],
	      (unsigned short int) scene->file[filenum]->battle[2].unknown2[i],
	      (unsigned short int) scene->file[filenum]->battle[3].unknown2[i]);
    }
    fprintf(fptr,"| End battle ID | [0x%04x]      | [0x%04x]      | [0x%04x]      | [0x%04x]      |\n",
	    (unsigned short int) scene->file[filenum]->battle[0].endBattleID,
	    (unsigned short int) scene->file[filenum]->battle[1].endBattleID,
	    (unsigned short int) scene->file[filenum]->battle[2].endBattleID,
	    (unsigned short int) scene->file[filenum]->battle[3].endBattleID);
    fprintf(fptr,"| BeginBattleID | [0x%04x]      | [0x%04x]      | [0x%04x]      | [0x%04x]      |\n",
	    (unsigned short int) scene->file[filenum]->battle[0].begBattleID,
	    (unsigned short int) scene->file[filenum]->battle[1].begBattleID,
	    (unsigned short int) scene->file[filenum]->battle[2].begBattleID,
	    (unsigned short int) scene->file[filenum]->battle[3].begBattleID);

    /* Battle camera IDs */
    fprintf(fptr,"*---------------*---------------*---------------*---------------*---------------*\n"); 
    fprintf(fptr,"| ========================= MONSTER CAMERA INFORMATION ======================== |\n");
    fprintf(fptr,"*-------------*----------*----------*----------*----------*----------*----------*\n");
    fprintf(fptr,"| ----------- | Value 1  | Value 2  | Value 3  | Value 4  | Value 5  | Value 6  |\n");
    fprintf(fptr,"*-------------*----------*----------*----------*----------*----------*----------*\n");

    for (j=0;j<4;j++) {
      for (i=0;i<4;i++) {
	fprintf(fptr,"| Scn.%d Mon.%d | [0x%04x] | [0x%04x] | [0x%04x] | [0x%04x] | [0x%04x] | [0x%04x] |\n",
		j,i,
		(unsigned short int) scene->file[filenum]->battle_cam[j].cameraData[i][0],
		(unsigned short int) scene->file[filenum]->battle_cam[j].cameraData[i][1],
		(unsigned short int) scene->file[filenum]->battle_cam[j].cameraData[i][2],
		(unsigned short int) scene->file[filenum]->battle_cam[j].cameraData[i][3],
		(unsigned short int) scene->file[filenum]->battle_cam[j].cameraData[i][4],
		(unsigned short int) scene->file[filenum]->battle_cam[j].cameraData[i][5]);
      }
      fprintf(fptr,"|             |          |          |          |          |          |          |\n");
    }


    /* Battle scenes specifications */
    fprintf(fptr,"*---------------*---------------*---------------*---------------*---------------*\n"); 
    fprintf(fptr,"| ======================== BATTLE SCENE SPECIFICATIONS ======================== |\n");
    fprintf(fptr,"*-------------------------------------------------------------------------------*\n");
    for (i=0;i<4;i++) {
      fprintf(fptr,"| ---------------------------------- SCENE %04d ------------------------------- |\n",
	      filenum*4+i);
      fprintf(fptr,"*-------------*----------*----------*----------*----------*----------*----------*\n");
      fprintf(fptr,"| MonsterID   | axis X   | axis Y   | axis Z   | line     | line (?) | undefined|\n");
      for (j=0;j<6;j++) {
	fprintf(fptr,"| [0x%04x]    | [%06d] | [%06d] | [%06d] | [%04d]   | [%04d]   | [%04d]   |\n",
		(unsigned short int) scene->file[filenum]->battle_spec[i][j].monsterID,
		scene->file[filenum]->battle_spec[i][j].axisX,
		scene->file[filenum]->battle_spec[i][j].axisY,
		scene->file[filenum]->battle_spec[i][j].axisZ,
		scene->file[filenum]->battle_spec[i][j].line,
		scene->file[filenum]->battle_spec[i][j].line2,
		scene->file[filenum]->battle_spec[i][j].undefined);
      }
      fprintf(fptr,"*-------------*----------*----------*----------*----------*----------*----------*\n");
    }

    /* Monster specifications */
    fprintf(fptr,"| =========================== MONSTER SPECIFICATIONS ========================== |\n");
    for (i=0;i<3;i++) {
      fprintf(fptr,"*-------------------------------------------------------------------------------*\n");
      text_format(scene->file[filenum]->monsterSpec[i].name,32);     
      fprintf(fptr,"| ID 0x%04x name=[%s",
	      scene->file[filenum]->monsterID[i],
	      scene->file[filenum]->monsterSpec[i].name);
      for(k=0;k<32-strlen((char *) scene->file[filenum]->monsterSpec[i].name);k++) { fprintf(fptr," "); }
      fprintf(fptr,"]                             |\n");
      fprintf(fptr,"|                                                                               |\n");
      text_restore(scene->file[filenum]->monsterSpec[i].name,32);     
      fprintf(fptr,"| Lvl [%03d] Spd [%03d] Lck [%03d] Esq [%03d] Str [%03d] Df [%03d] Mg [%03d] DefM [%03d]|\n",
	      scene->file[filenum]->monsterSpec[i].main_carac.level,
	      scene->file[filenum]->monsterSpec[i].main_carac.speed,
	      scene->file[filenum]->monsterSpec[i].main_carac.luck,
	      scene->file[filenum]->monsterSpec[i].main_carac.physic_esq,
	      scene->file[filenum]->monsterSpec[i].main_carac.strength,
	      scene->file[filenum]->monsterSpec[i].main_carac.physic_def,
	      scene->file[filenum]->monsterSpec[i].main_carac.magi,
	      scene->file[filenum]->monsterSpec[i].main_carac.magic_def);
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"|              Element                  |   Link between monster and element    |\n");
      for (j=0;j<8;j++) {
	buffer = element_name(scene->file[filenum]->monsterSpec[i].elements[j]);
	fprintf(fptr,"| [0x%02x] %s  ",
		(unsigned char) scene->file[filenum]->monsterSpec[i].elements[j],
		buffer);
	for(k=0;k<29-strlen(buffer);k++) { fprintf(fptr," "); }
	free(buffer);

	buffer = link_name(scene->file[filenum]->monsterSpec[i].links[j]);
	fprintf(fptr,"| [0x%02x] %s  ",
		(unsigned char) scene->file[filenum]->monsterSpec[i].links[j],
		buffer);
	for(k=0;k<29-strlen(buffer);k++) { fprintf(fptr," "); }
	fprintf(fptr,"|\n");
	free(buffer);
      }
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"| attack ID                | attack text ID           | camera ID               |\n");
      for (j=0;j<16;j++) {
	fprintf(fptr,"| [0x%02x]                   | [0x%04x]                 | [0x%04x]                |\n",
		(unsigned char) scene->file[filenum]->monsterSpec[i].actionID[j],
		(unsigned short int) scene->file[filenum]->monsterSpec[i].actionTextID[j],
		(unsigned short int) scene->file[filenum]->monsterSpec[i].cameraID[j]);
      }
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"| Method and probability to obtain it |                Object ID                |\n");
      for (j=0;j<4;j++) {
	buffer = object_name(scene->file[filenum]->monsterSpec[i].object[j]);
	fprintf(fptr,"| [0x%02x]                              | [0x%04x] %s",
		(unsigned char) scene->file[filenum]->monsterSpec[i].link_object[j],
		(unsigned short int) scene->file[filenum]->monsterSpec[i].object[j],
		buffer);
	for(k=0;k<31-strlen(buffer);k++) { fprintf(fptr," "); }
	fprintf(fptr,"|\n");
	free(buffer);
      }
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"| IDs of attacks player can do when manipulated : [0x%04x] [0x%04x] [0x%04x]    |\n",
	      (unsigned short int) scene->file[filenum]->monsterSpec[i].manipAttackID[0],
	      (unsigned short int) scene->file[filenum]->monsterSpec[i].manipAttackID[1],
	      (unsigned short int) scene->file[filenum]->monsterSpec[i].manipAttackID[2]);
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"| I don t remind why, but we called this param. attackProbability (?) [0x%04x]  |\n",
	      (unsigned short int) scene->file[filenum]->monsterSpec[i].attackProbability);
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"| MP [%05d] AP [%05d]      ID of object obtained with Trans. materia [0x%04x] |\n",
	      scene->file[filenum]->monsterSpec[i].MP,
	      scene->file[filenum]->monsterSpec[i].AP,
	      (unsigned short int) scene->file[filenum]->monsterSpec[i].trans_object);
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"| unknown [0x%02x] [0x%02x]   HP [%010d]  EXP[%010d]  gils [%010d]   |\n",
	      scene->file[filenum]->monsterSpec[i].unknown2[0],
	      scene->file[filenum]->monsterSpec[i].unknown2[1],
	      scene->file[filenum]->monsterSpec[i].HP,
	      scene->file[filenum]->monsterSpec[i].EXP,
	      scene->file[filenum]->monsterSpec[i].gils);
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"|                              ---- FLAGS ----                                  |\n");
      for (j=0;j<32;j++) {
	buffer = flag_name(j);
	fprintf(fptr,"| [%d] %s",
		((scene->file[filenum]->monsterSpec[i].flags[j/8] & ((unsigned char) (1 << (7-(j%8)) ))) == 0)?0:1,
		buffer);
	for(k=0;k<33-strlen(buffer);k++) { fprintf(fptr," "); }
	if (j%2 == 1) {
	  fprintf(fptr," |\n");
	} else {
	  fprintf(fptr," ");
	}
	free(buffer);
      }
      fprintf(fptr,"|                                                                               |\n");
      fprintf(fptr,"| unknown [0x%02x] [0x%02x] [0x%02x] [0x%02x]                                           |\n",
	      scene->file[filenum]->monsterSpec[i].unknown4[0],
	      scene->file[filenum]->monsterSpec[i].unknown4[1],
	      scene->file[filenum]->monsterSpec[i].unknown4[2],
	      scene->file[filenum]->monsterSpec[i].unknown4[3]);
    }

    /* Attack data */
    fprintf(fptr,"*-------------------------------------------------------------------------------*\n");
    fprintf(fptr,"| ================================ ATTACK DATA ================================ |\n");
    fprintf(fptr,"*----*--------------------------------------------------------------------------*\n");
    fprintf(fptr,"| NN | Has something to do with attacks (28 bytes MUST be in each field)        |\n");
    fprintf(fptr,"*----*--------------------------------------------------------------------------*-------------*\n");

    for (k=0;k<32;k++) {
      fprintf(fptr,"| %02d | [",k);
      for (j=0;j<28;j++) {
	fprintf(fptr,"%02x ",scene->file[filenum]->attacks.unknownAtt[k][j]);
      }
      fprintf(fptr,"] |\n");
    }

    fprintf(fptr,"*----*-----*--------------------------------------------------------------------*-------------*\n");
    fprintf(fptr,"|  NameID  | Attack Name                                                        |\n");
    fprintf(fptr,"*----------*--------------------------------------------------------------------*\n");
    
    for (k=0;k<32;k++) {
      text_format(scene->file[filenum]->attacks.attackName[k],32);
      fprintf(fptr,"| [0x%04x] | [%s",
	      (unsigned short int) scene->file[filenum]->attacks.nameID[k],
	      scene->file[filenum]->attacks.attackName[k]);
      for (j=0;j<32-strlen((char *) scene->file[filenum]->attacks.attackName[k]);j++) { fprintf(fptr," "); }
      fprintf(fptr,"]");
      fprintf(fptr,"                                 |\n");
      text_restore(scene->file[filenum]->attacks.attackName[k],32);
    }
    
    fprintf(fptr,"*----------*--------------------------------------------------------------------*\n");
    fprintf(fptr,"| Unknown data structure                                                        |\n");
    fprintf(fptr,"*-------------------------------------------------------------------------------*\n");

    for (k=0;k<16;k++) {
      fprintf(fptr,"| [");
      for (j=0;j<16;j++) {
	fprintf(fptr,"%02x ",scene->file[filenum]->attacks.unknown[k*32+j]);
      }
      fprintf(fptr,"]                            |\n");
      fprintf(fptr,"| [");
      for (j=16;j<32;j++) {
	fprintf(fptr,"%02x ",scene->file[filenum]->attacks.unknown[k*32+j]);
      }
      fprintf(fptr,"]                            |\n");
    }
    
    /* Scripts */
    fprintf(fptr,"*-------------------------------------------------------------------------------*\n");
    fprintf(fptr,"| ================================== SCRIPTS ================================== |\n");
    fprintf(fptr,"*-------------------------------------------------------------------------------*\n");
    fprintf(fptr,"NOTES : (1) Empty fields must NOT be erased.  (2) Hex prefix \"0x\" is implicit.\n");
    fprintf(fptr,"        (3) String can be written between double quotes \"\n\n");
    for (i=0;i<3;i++) {
      for (j=0;j<16;j++) {
	if (scene->file[filenum]->scripts.pointer[i] == -1) {
	  fprintf(fptr,"Block %02d SubBlock %02d []\n",i,j);

	} else {
	  if (scene->file[filenum]->scripts.block[i].subBlockSize[j] <= 0) {
	    fprintf(fptr,"Block %02d SubBlock %02d []\n",i,j);

	  } else {
	    cr=0;
	    
	    fprintf(fptr,"Block %02d SubBlock %02d\n[",i,j);
	    for (k=0;k<scene->file[filenum]->scripts.block[i].subBlockSize[j];k++) {
	      fprintf(fptr,"%02x ",scene->file[filenum]->scripts.block[i].subBlock[j][k]);
	      cr++;
	      if (scene->file[filenum]->scripts.block[i].subBlock[j][k] == 0x72) {
		cr=0;
		fprintf(fptr,"\n ");
	      
	      } else if (scene->file[filenum]->scripts.block[i].subBlock[j][k] == 0x93) { /* Text */
		if (k > 0) {
		  if (scene->file[filenum]->scripts.block[i].subBlock[j][k-1] < 0x60 ||
		      scene->file[filenum]->scripts.block[i].subBlock[j][k-1] > 0x73) {
		    fprintf(fptr,"\"");
		    k++;
		    /* FF7 strings always finish by a 0xFF (I hope ^^) */
		    while (scene->file[filenum]->scripts.block[i].subBlock[j][k] != 0xFF) {
		      text_format(&scene->file[filenum]->scripts.block[i].subBlock[j][k],1);
		      text_restore(&scene->file[filenum]->scripts.block[i].subBlock[j][k],1);
		      text_format(&scene->file[filenum]->scripts.block[i].subBlock[j][k],1);
		      fprintf(fptr,"%c",scene->file[filenum]->scripts.block[i].subBlock[j][k]);
		      text_restore(&scene->file[filenum]->scripts.block[i].subBlock[j][k],1);
		      k++;
		    }
		    fprintf(fptr,"\"\n ");
		    cr=0;
		  }
		}
		    
	      } else if (cr > 31) {
		fprintf(fptr,"\n ");
		cr=0;
	      }
	      
	    } /* End of for (k) */

	    fprintf(fptr,"]\n\n");

	  }
	}
      } /* End of for (j) */
    } /* End of for (i) */
    

  }

  fclose(fptr);

  return 0;
}

/* *** Converts a string to a readable format *** */

void text_format(unsigned char *str, int length) {
  int i=0;

  while (i < length) {

    if (str[i] < 0x60) {
      str[i]+=0x20;
    } else if (str[i] <= 0x85) {
      str[i]+=0x7A;
    } else if (str[i] == 0xFF) {
      str[i] = 0;
    }

    /* Usually for french accents */
    if (str[i] == 0xE8) {
      str[i] = 0xE9;
    } else if (str[i] == 0xF3) {
      str[i] = 0xF4;
    } else if (str[i] == 0xE3) {
      str[i] = 0xE2;
    } else if (str[i] == 0xE9) {
      str[i] = 0xE8;
    } else if (str[i] == 0xF8) {
      str[i] = 0xFB;
    } else if (str[i] == 0xE2) {
      str[i] = 0xE0;
    } else if (str[i] == 0xDC) {
      str[i] = 0xE7;
    }
    
    if (str[i] == 0xA8) {
      str[i] = 0xBB;
    } else if (str[i] == 0xA7) {
      str[i] = 0xAB;
    } else if (str[i] == 0xA9) {
      str[i] = 0xBA;
    } else if (str[i] == 0x22) { /* Removing 'special' double quotes */
      str[i] = 0x84;
    }

    i++;
  }

  return;
}

/* *** Converts a string to the "FF7 format" *** */

void text_restore(unsigned char *str, int length) {
  int i=0;
 
  while (i<length) {

    /* Reverse conversion of accents */
    if (str[i] == 0xE9) {
      str[i] = 0xE8;
    } else if (str[i] == 0xF4) {
      str[i] = 0xF3;
    } else if (str[i] == 0xE2) {
      str[i] = 0xE3;
    } else if (str[i] == 0xE8) {
      str[i] = 0xE9;
    } else if (str[i] == 0xFB) {
      str[i] = 0xF8;
    } else if (str[i] == 0xE0) {
      str[i] = 0xE2;
    } else if (str[i] == 0xE7) {
      str[i] = 0xDC;
    } else if (str[i] == 0x84) {
      str[i] = 0x22;
    }

    if (str[i]==0) {
      str[i]=0xFF;
    } else if (str[i]>=0xDA) {
      str[i]-=0x7A;
    } else if (str[i] >= 0x20 && str[i] < 0x80) {
      str[i]-=0x20;
    }

    if (str[i] == 0xBB) {
      str[i] = 0xA8;
    } else if (str[i] == 0xAB) {
      str[i] = 0xA7;
    } else if (str[i] == 0xBA) {
      str[i] = 0xA9;
    }

    i++;
  }
  
  return;
}

/* *** Returns an object name from its ID number *** */

char *object_name(int val) { 

  char *objName;
  objName=(char*) malloc(sizeof(char)*40);

  if (val == 0x000) sprintf(objName,OBJECT_000);
  else if (val == 0x001) sprintf(objName,OBJECT_001);
  else if (val == 0x002) sprintf(objName,OBJECT_002);                     
  else if (val == 0x003) sprintf(objName,OBJECT_003);                        
  else if (val == 0x004) sprintf(objName,OBJECT_004);                  
  else if (val == 0x005) sprintf(objName,OBJECT_005);                       
  else if (val == 0x006) sprintf(objName,OBJECT_006);                    
  else if (val == 0x007) sprintf(objName,OBJECT_007);                 
  else if (val == 0x008) sprintf(objName,OBJECT_008);                     
  else if (val == 0x009) sprintf(objName,OBJECT_009);                         
  else if (val == 0x00A) sprintf(objName,OBJECT_00A);                
  else if (val == 0x00B) sprintf(objName,OBJECT_00B);                   
  else if (val == 0x00C) sprintf(objName,OBJECT_00C);                  
  else if (val == 0x00D) sprintf(objName,OBJECT_00D);                        
  else if (val == 0x00E) sprintf(objName,OBJECT_00E);                 
  else if (val == 0x00F) sprintf(objName,OBJECT_00F);                       
  else if (val == 0x010) sprintf(objName,OBJECT_010);                   
  else if (val == 0x011) sprintf(objName,OBJECT_011);                  
  else if (val == 0x012) sprintf(objName,OBJECT_012);                   
  else if (val == 0x013) sprintf(objName,OBJECT_013);                      
  else if (val == 0x014) sprintf(objName,OBJECT_014);                      
  else if (val == 0x015) sprintf(objName,OBJECT_015); 
  else if (val == 0x016) sprintf(objName,OBJECT_016);                    
  else if (val == 0x017) sprintf(objName,OBJECT_017);                    
  else if (val == 0x018) sprintf(objName,OBJECT_018);                
  else if (val == 0x019) sprintf(objName,OBJECT_019);                   
  else if (val == 0x01A) sprintf(objName,OBJECT_01A);                 
  else if (val == 0x01B) sprintf(objName,OBJECT_01B);                    
  else if (val == 0x01C) sprintf(objName,OBJECT_01C);                     
  else if (val == 0x01D) sprintf(objName,OBJECT_01D);                    
  else if (val == 0x01E) sprintf(objName,OBJECT_01E);                    
  else if (val == 0x01F) sprintf(objName,OBJECT_01F);                    
  else if (val == 0x020) sprintf(objName,OBJECT_020);               
  else if (val == 0x021) sprintf(objName,OBJECT_021);                  
  else if (val == 0x022) sprintf(objName,OBJECT_022);                   
  else if (val == 0x023) sprintf(objName,OBJECT_023);                   
  else if (val == 0x024) sprintf(objName,OBJECT_024);                   
  else if (val == 0x025) sprintf(objName,OBJECT_025);                 
  else if (val == 0x026) sprintf(objName,OBJECT_026);                 
  else if (val == 0x027) sprintf(objName,OBJECT_027);                  
  else if (val == 0x028) sprintf(objName,OBJECT_028);                     
  else if (val == 0x029) sprintf(objName,OBJECT_029);                      
  else if (val == 0x02A) sprintf(objName,OBJECT_02A);                        
  else if (val == 0x02B) sprintf(objName,OBJECT_02B);                     
  else if (val == 0x02C) sprintf(objName,OBJECT_02C);                     
  else if (val == 0x02D) sprintf(objName,OBJECT_02D);                     
  else if (val == 0x02E) sprintf(objName,OBJECT_02E);                            
  else if (val == 0x02F) sprintf(objName,OBJECT_02F);                        
  else if (val == 0x030) sprintf(objName,OBJECT_030);                         
  else if (val == 0x031) sprintf(objName,OBJECT_031);                     
  else if (val == 0x032) sprintf(objName,OBJECT_032);                           
  else if (val == 0x033) sprintf(objName,OBJECT_033);                           
  else if (val == 0x034) sprintf(objName,OBJECT_034);                          
  else if (val == 0x035) sprintf(objName,OBJECT_035);                           
  else if (val == 0x036) sprintf(objName,OBJECT_036);                            
  else if (val == 0x037) sprintf(objName,OBJECT_037);                      
  else if (val == 0x038) sprintf(objName,OBJECT_038);                         
  else if (val == 0x039) sprintf(objName,OBJECT_039);                          
  else if (val == 0x03A) sprintf(objName,OBJECT_03A);                               
  else if (val == 0x03B) sprintf(objName,OBJECT_03B);                            
  else if (val == 0x03C) sprintf(objName,OBJECT_03C);                       
  else if (val == 0x03D) sprintf(objName,OBJECT_03D);                          
  else if (val == 0x03E) sprintf(objName,OBJECT_03E);                     
  else if (val == 0x03F) sprintf(objName,OBJECT_03F);                     
  else if (val == 0x040) sprintf(objName,OBJECT_040);                     
  else if (val == 0x041) sprintf(objName,OBJECT_041);                     
  else if (val == 0x042) sprintf(objName,OBJECT_042);                    
  else if (val == 0x043) sprintf(objName,OBJECT_043);                     
  else if (val == 0x044) sprintf(objName,OBJECT_044);                     
  else if (val == 0x045) sprintf(objName,OBJECT_045);                     
  else if (val == 0x046) sprintf(objName,OBJECT_046);                              
  else if (val == 0x047) sprintf(objName,OBJECT_047);                      
  else if (val == 0x048) sprintf(objName,OBJECT_048);                      
  else if (val == 0x049) sprintf(objName,OBJECT_049);                      
  else if (val == 0x04A) sprintf(objName,OBJECT_04A);                       
  else if (val == 0x04B) sprintf(objName,OBJECT_04B);                      
  else if (val == 0x04C) sprintf(objName,OBJECT_04C);                       
  else if (val == 0x04D) sprintf(objName,OBJECT_04D);                          
  else if (val == 0x04E) sprintf(objName,OBJECT_04E);                         
  else if (val == 0x04F) sprintf(objName,OBJECT_04F);                         
  else if (val == 0x050) sprintf(objName,OBJECT_050);                          
  else if (val == 0x051) sprintf(objName,OBJECT_051);                         
  else if (val == 0x052) sprintf(objName,OBJECT_052);                        
  else if (val == 0x053) sprintf(objName,OBJECT_053);                       
  else if (val == 0x054) sprintf(objName,OBJECT_054);                         
  else if (val == 0x055) sprintf(objName,OBJECT_055);                           
  else if (val == 0x056) sprintf(objName,OBJECT_056);                            
  else if (val == 0x057) sprintf(objName,OBJECT_057);                         
  else if (val == 0x058) sprintf(objName,OBJECT_058);                       
  else if (val == 0x059) sprintf(objName,OBJECT_059);                      
  else if (val == 0x05A) sprintf(objName,OBJECT_05A);                      
  else if (val == 0x05B) sprintf(objName,OBJECT_05B);                      
  else if (val == 0x05C) sprintf(objName,OBJECT_05C);                      
  else if (val == 0x05D) sprintf(objName,OBJECT_05D);                             
  else if (val == 0x05E) sprintf(objName,OBJECT_05E);                          
  else if (val == 0x05F) sprintf(objName,OBJECT_05F);                      
  else if (val == 0x060) sprintf(objName,OBJECT_060); 
  else if (val == 0x061) sprintf(objName,OBJECT_061);
  else if (val == 0x062) sprintf(objName,OBJECT_062); 
  else if (val == 0x063) sprintf(objName,OBJECT_063); 
  else if (val == 0x064) sprintf(objName,OBJECT_064);    
  else if (val == 0x065) sprintf(objName,OBJECT_065);      
  else if (val == 0x066) sprintf(objName,OBJECT_066);  
  else if (val == 0x067) sprintf(objName,OBJECT_067);   
  else if (val == 0x068) sprintf(objName,OBJECT_068);   
  else if (val == 0x080) sprintf(objName,OBJECT_080);   
  else if (val == 0x081) sprintf(objName,OBJECT_081);     
  else if (val == 0x082) sprintf(objName,OBJECT_082);          
  else if (val == 0x083) sprintf(objName,OBJECT_083);    
  else if (val == 0x084) sprintf(objName,OBJECT_084);     
  else if (val == 0x085) sprintf(objName,OBJECT_085);          
  else if (val == 0x086) sprintf(objName,OBJECT_086);     
  else if (val == 0x087) sprintf(objName,OBJECT_087);     
  else if (val == 0x088) sprintf(objName,OBJECT_088);        
  else if (val == 0x089) sprintf(objName,OBJECT_089);          
  else if (val == 0x08A) sprintf(objName,OBJECT_08A);          
  else if (val == 0x08B) sprintf(objName,OBJECT_08B);         
  else if (val == 0x08C) sprintf(objName,OBJECT_08C);        
  else if (val == 0x08D) sprintf(objName,OBJECT_08D);    
  else if (val == 0x08E) sprintf(objName,OBJECT_08E);          
  else if (val == 0x08F) sprintf(objName,OBJECT_08F);     
  else if (val == 0x090) sprintf(objName,OBJECT_090);     
  else if (val == 0x091) sprintf(objName,OBJECT_091);     
  else if (val == 0x092) sprintf(objName,OBJECT_092);                     
  else if (val == 0x093) sprintf(objName,OBJECT_093);                       
  else if (val == 0x094) sprintf(objName,OBJECT_094);                        
  else if (val == 0x095) sprintf(objName,OBJECT_095);                   
  else if (val == 0x096) sprintf(objName,OBJECT_096);                       
  else if (val == 0x097) sprintf(objName,OBJECT_097);                     
  else if (val == 0x098) sprintf(objName,OBJECT_098);                       
  else if (val == 0x099) sprintf(objName,OBJECT_099);                     
  else if (val == 0x09A) sprintf(objName,OBJECT_09A);                    
  else if (val == 0x09B) sprintf(objName,OBJECT_09B);                        
  else if (val == 0x09C) sprintf(objName,OBJECT_09C);                         
  else if (val == 0x09D) sprintf(objName,OBJECT_09D);                       
  else if (val == 0x09E) sprintf(objName,OBJECT_09E);                        
  else if (val == 0x09F) sprintf(objName,OBJECT_09F);                     
  else if (val == 0x0A0) sprintf(objName,OBJECT_0A0);
  else if (val == 0x0A1) sprintf(objName,OBJECT_0A1);
  else if (val == 0x0A2) sprintf(objName,OBJECT_0A2);
  else if (val == 0x0A3) sprintf(objName,OBJECT_0A3);
  else if (val == 0x0A4) sprintf(objName,OBJECT_0A4);
  else if (val == 0x0A5) sprintf(objName,OBJECT_0A5);
  else if (val == 0x0A6) sprintf(objName,OBJECT_0A6);
  else if (val == 0x0A7) sprintf(objName,OBJECT_0A7);
  else if (val == 0x0A8) sprintf(objName,OBJECT_0A8);
  else if (val == 0x0A9) sprintf(objName,OBJECT_0A9);
  else if (val == 0x0AA) sprintf(objName,OBJECT_0AA);
  else if (val == 0x0AB) sprintf(objName,OBJECT_0AB);
  else if (val == 0x0AC) sprintf(objName,OBJECT_0AC);
  else if (val == 0x0AD) sprintf(objName,OBJECT_0AD);
  else if (val == 0x0AE) sprintf(objName,OBJECT_0AE);
  else if (val == 0x0AF) sprintf(objName,OBJECT_0AF);
  else if (val == 0x0B0) sprintf(objName,OBJECT_0B0);
  else if (val == 0x0B1) sprintf(objName,OBJECT_0B1);
  else if (val == 0x0B2) sprintf(objName,OBJECT_0B2);
  else if (val == 0x0B3) sprintf(objName,OBJECT_0B3);
  else if (val == 0x0B4) sprintf(objName,OBJECT_0B4);
  else if (val == 0x0B5) sprintf(objName,OBJECT_0B5);
  else if (val == 0x0B6) sprintf(objName,OBJECT_0B6);
  else if (val == 0x0B7) sprintf(objName,OBJECT_0B7);
  else if (val == 0x0B8) sprintf(objName,OBJECT_0B8);
  else if (val == 0x0B9) sprintf(objName,OBJECT_0B9);
  else if (val == 0x0BA) sprintf(objName,OBJECT_0BA);
  else if (val == 0x0BB) sprintf(objName,OBJECT_0BB);
  else if (val == 0x0BC) sprintf(objName,OBJECT_0BC);
  else if (val == 0x0BD) sprintf(objName,OBJECT_0BD);
  else if (val == 0x0BE) sprintf(objName,OBJECT_0BE);
  else if (val == 0x0BF) sprintf(objName,OBJECT_0BF);
  else if (val == 0x0C0) sprintf(objName,OBJECT_0C0);
  else if (val == 0x0C1) sprintf(objName,OBJECT_0C1);
  else if (val == 0x0C2) sprintf(objName,OBJECT_0C2);
  else if (val == 0x0C3) sprintf(objName,OBJECT_0C3);
  else if (val == 0x0C4) sprintf(objName,OBJECT_0C4);
  else if (val == 0x0C5) sprintf(objName,OBJECT_0C5);
  else if (val == 0x0C6) sprintf(objName,OBJECT_0C6);
  else if (val == 0x0C7) sprintf(objName,OBJECT_0C7);
  else if (val == 0x0C8) sprintf(objName,OBJECT_0C8);
  else if (val == 0x0C9) sprintf(objName,OBJECT_0C9);
  else if (val == 0x0CA) sprintf(objName,OBJECT_0CA);
  else if (val == 0x0CB) sprintf(objName,OBJECT_0CB);
  else if (val == 0x0CC) sprintf(objName,OBJECT_0CC);
  else if (val == 0x0CD) sprintf(objName,OBJECT_0CD);
  else if (val == 0x0CE) sprintf(objName,OBJECT_0CE);
  else if (val == 0x0CF) sprintf(objName,OBJECT_0CF);
  else if (val == 0x0D0) sprintf(objName,OBJECT_0D0);
  else if (val == 0x0D1) sprintf(objName,OBJECT_0D1);
  else if (val == 0x0D2) sprintf(objName,OBJECT_0D2);
  else if (val == 0x0D3) sprintf(objName,OBJECT_0D3);
  else if (val == 0x0D4) sprintf(objName,OBJECT_0D4);
  else if (val == 0x0D5) sprintf(objName,OBJECT_0D5);
  else if (val == 0x0D6) sprintf(objName,OBJECT_0D6);
  else if (val == 0x0D7) sprintf(objName,OBJECT_0D7);
  else if (val == 0x0D8) sprintf(objName,OBJECT_0D8);
  else if (val == 0x0D9) sprintf(objName,OBJECT_0D9);
  else if (val == 0x0DA) sprintf(objName,OBJECT_0DA);
  else if (val == 0x0DB) sprintf(objName,OBJECT_0DB);
  else if (val == 0x0DC) sprintf(objName,OBJECT_0DC);
  else if (val == 0x0DD) sprintf(objName,OBJECT_0DD);
  else if (val == 0x0DE) sprintf(objName,OBJECT_0DE);
  else if (val == 0x0DF) sprintf(objName,OBJECT_0DF);
  else if (val == 0x0E0) sprintf(objName,OBJECT_0E0);
  else if (val == 0x0E1) sprintf(objName,OBJECT_0E1);
  else if (val == 0x0E2) sprintf(objName,OBJECT_0E2);
  else if (val == 0x0E3) sprintf(objName,OBJECT_0E3);
  else if (val == 0x0E4) sprintf(objName,OBJECT_0E4);
  else if (val == 0x0E5) sprintf(objName,OBJECT_0E5);
  else if (val == 0x0E6) sprintf(objName,OBJECT_0E6);
  else if (val == 0x0E7) sprintf(objName,OBJECT_0E7);
  else if (val == 0x0E8) sprintf(objName,OBJECT_0E8);
  else if (val == 0x0E9) sprintf(objName,OBJECT_0E9);
  else if (val == 0x0EA) sprintf(objName,OBJECT_0EA);
  else if (val == 0x0EB) sprintf(objName,OBJECT_0EB);
  else if (val == 0x0EC) sprintf(objName,OBJECT_0EC);
  else if (val == 0x0ED) sprintf(objName,OBJECT_0ED);
  else if (val == 0x0EE) sprintf(objName,OBJECT_0EE);
  else if (val == 0x0EF) sprintf(objName,OBJECT_0EF);
  else if (val == 0x0F0) sprintf(objName,OBJECT_0F0);
  else if (val == 0x0F1) sprintf(objName,OBJECT_0F1);
  else if (val == 0x0F2) sprintf(objName,OBJECT_0F2);
  else if (val == 0x0F3) sprintf(objName,OBJECT_0F3);
  else if (val == 0x0F4) sprintf(objName,OBJECT_0F4);
  else if (val == 0x0F5) sprintf(objName,OBJECT_0F5);
  else if (val == 0x0F6) sprintf(objName,OBJECT_0F6);
  else if (val == 0x0F7) sprintf(objName,OBJECT_0F7);
  else if (val == 0x0F8) sprintf(objName,OBJECT_0F8);
  else if (val == 0x0F9) sprintf(objName,OBJECT_0F9);
  else if (val == 0x0FA) sprintf(objName,OBJECT_0FA);
  else if (val == 0x0FB) sprintf(objName,OBJECT_0FB);
  else if (val == 0x0FC) sprintf(objName,OBJECT_0FC);
  else if (val == 0x0FD) sprintf(objName,OBJECT_0FD);
  else if (val == 0x0FE) sprintf(objName,OBJECT_0FE);
  else if (val == 0x0FF) sprintf(objName,OBJECT_0FF);
  else if (val == 0x100) sprintf(objName,OBJECT_100);
  else if (val == 0x101) sprintf(objName,OBJECT_101);                  
  else if (val == 0x102) sprintf(objName,OBJECT_102);                 
  else if (val == 0x103) sprintf(objName,OBJECT_103);               
  else if (val == 0x104) sprintf(objName,OBJECT_104);                
  else if (val == 0x105) sprintf(objName,OBJECT_105);                
  else if (val == 0x106) sprintf(objName,OBJECT_106);                  
  else if (val == 0x107) sprintf(objName,OBJECT_107);               
  else if (val == 0x108) sprintf(objName,OBJECT_108);              
  else if (val == 0x109) sprintf(objName,OBJECT_109);              
  else if (val == 0x10A) sprintf(objName,OBJECT_10A);                 
  else if (val == 0x10B) sprintf(objName,OBJECT_10B);                     
  else if (val == 0x10C) sprintf(objName,OBJECT_10C);              
  else if (val == 0x10D) sprintf(objName,OBJECT_10D);
  else if (val == 0x10E) sprintf(objName,OBJECT_10E);
  else if (val == 0x10F) sprintf(objName,OBJECT_10F);
  else if (val == 0x110) sprintf(objName,OBJECT_110);
  else if (val == 0x111) sprintf(objName,OBJECT_111);
  else if (val == 0x112) sprintf(objName,OBJECT_112);
  else if (val == 0x113) sprintf(objName,OBJECT_113);
  else if (val == 0x114) sprintf(objName,OBJECT_114);
  else if (val == 0x115) sprintf(objName,OBJECT_115);
  else if (val == 0x116) sprintf(objName,OBJECT_116);
  else if (val == 0x117) sprintf(objName,OBJECT_117);
  else if (val == 0x118) sprintf(objName,OBJECT_118);
  else if (val == 0x119) sprintf(objName,OBJECT_119);
  else if (val == 0x11A) sprintf(objName,OBJECT_11A);
  else if (val == 0x11B) sprintf(objName,OBJECT_11B);
  else if (val == 0x11C) sprintf(objName,OBJECT_11C);
  else if (val == 0x11D) sprintf(objName,OBJECT_11D);
  else if (val == 0x11E) sprintf(objName,OBJECT_11E);
  else if (val == 0x11F) sprintf(objName,OBJECT_11F);
  else if (val == 0x120) sprintf(objName,OBJECT_120);
  else if (val == 0x121) sprintf(objName,OBJECT_121);
  else if (val == 0x122) sprintf(objName,OBJECT_122);
  else if (val == 0x123) sprintf(objName,OBJECT_123);
  else if (val == 0x124) sprintf(objName,OBJECT_124);
  else if (val == 0x125) sprintf(objName,OBJECT_125);
  else if (val == 0x126) sprintf(objName,OBJECT_126);
  else if (val == 0x127) sprintf(objName,OBJECT_127);
  else if (val == 0x128) sprintf(objName,OBJECT_128);
  else if (val == 0x129) sprintf(objName,OBJECT_129);
  else if (val == 0x12A) sprintf(objName,OBJECT_12A);
  else if (val == 0x12B) sprintf(objName,OBJECT_12B);
  else if (val == 0x12C) sprintf(objName,OBJECT_12C);
  else if (val == 0x12D) sprintf(objName,OBJECT_12D);
  else if (val == 0x12E) sprintf(objName,OBJECT_12E);
  else if (val == 0x12F) sprintf(objName,OBJECT_12F);
  else if (val == 0x130) sprintf(objName,OBJECT_130);
  else if (val == 0x131) sprintf(objName,OBJECT_131);
  else if (val == 0x132) sprintf(objName,OBJECT_132);
  else if (val == 0x133) sprintf(objName,OBJECT_133);
  else if (val == 0x134) sprintf(objName,OBJECT_134);
  else if (val == 0x135) sprintf(objName,OBJECT_135);
  else if (val == 0x136) sprintf(objName,OBJECT_136);
  else if (val == 0x137) sprintf(objName,OBJECT_137);
  else if (val == 0x138) sprintf(objName,OBJECT_138);
  else if (val == 0x139) sprintf(objName,OBJECT_139);
  else if (val == 0x13A) sprintf(objName,OBJECT_13A);
  else if (val == 0x13B) sprintf(objName,OBJECT_13B);
  else if (val == 0x13C) sprintf(objName,OBJECT_13C);
  else if (val == 0x13D) sprintf(objName,OBJECT_13D);
  else if (val == 0x13E) sprintf(objName,OBJECT_13E);
  else if (val == 0x13F) sprintf(objName,OBJECT_13F);
  else sprintf(objName,OBJECT_FFF);

  return objName;
}

/* *** Returns an element name from its ID number *** */

char *element_name(int val) { 

  char *elementName;
  elementName=(char*) malloc(sizeof(char)*40);

  if (val == 0x000) sprintf(elementName,ELEMENT_000);
  else if (val == 0x001) sprintf(elementName,ELEMENT_001);
  else if (val == 0x002) sprintf(elementName,ELEMENT_002);
  else if (val == 0x003) sprintf(elementName,ELEMENT_003);
  else if (val == 0x004) sprintf(elementName,ELEMENT_004);
  else if (val == 0x005) sprintf(elementName,ELEMENT_005);
  else if (val == 0x006) sprintf(elementName,ELEMENT_006);
  else if (val == 0x007) sprintf(elementName,ELEMENT_007);
  else if (val == 0x008) sprintf(elementName,ELEMENT_008);
  else if (val == 0x009) sprintf(elementName,ELEMENT_009);
  else if (val == 0x00a) sprintf(elementName,ELEMENT_00A);
  else if (val == 0x00b) sprintf(elementName,ELEMENT_00B);
  else if (val == 0x00c) sprintf(elementName,ELEMENT_00C);
  else if (val == 0x00d) sprintf(elementName,ELEMENT_00D);
  else if (val == 0x00e) sprintf(elementName,ELEMENT_00E);
  else if (val == 0x00f) sprintf(elementName,ELEMENT_00F);
  else sprintf(elementName,ELEMENT_FFF);

  return elementName;
}

/* *** Returns a link name from its ID number *** */

char *link_name(int val) { 

  char *linkName;
  linkName=(char*) malloc(sizeof(char)*40);

  if (val == 0x000) sprintf(linkName,LINK_000);
  else if (val == 0x001) sprintf(linkName,LINK_001);
  else if (val == 0x002) sprintf(linkName,LINK_002);
  else if (val == 0x003) sprintf(linkName,LINK_003);
  else if (val == 0x004) sprintf(linkName,LINK_004);
  else if (val == 0x005) sprintf(linkName,LINK_005);
  else if (val == 0x006) sprintf(linkName,LINK_006);
  else if (val == 0x007) sprintf(linkName,LINK_007);
  else sprintf(linkName,LINK_FFF);

  return linkName;
}

/* *** Returns name of a flag *** */

char *flag_name(int val) { 

  char *flagName;
  flagName=(char*) malloc(sizeof(char)*40);

  if (val == 0x000) sprintf(flagName,FLAG_000);
  else if (val == 0x001) sprintf(flagName,FLAG_001);
  else if (val == 0x002) sprintf(flagName,FLAG_002);
  else if (val == 0x003) sprintf(flagName,FLAG_003);
  else if (val == 0x004) sprintf(flagName,FLAG_004);
  else if (val == 0x005) sprintf(flagName,FLAG_005);
  else if (val == 0x006) sprintf(flagName,FLAG_006);
  else if (val == 0x007) sprintf(flagName,FLAG_007);
  else if (val == 0x008) sprintf(flagName,FLAG_008);
  else if (val == 0x009) sprintf(flagName,FLAG_009);
  else if (val == 0x00A) sprintf(flagName,FLAG_00A);
  else if (val == 0x00B) sprintf(flagName,FLAG_00B);
  else if (val == 0x00C) sprintf(flagName,FLAG_00C);
  else if (val == 0x00D) sprintf(flagName,FLAG_00D);
  else if (val == 0x00E) sprintf(flagName,FLAG_00E);
  else if (val == 0x00F) sprintf(flagName,FLAG_00F);
  else if (val == 0x010) sprintf(flagName,FLAG_010);
  else if (val == 0x011) sprintf(flagName,FLAG_011);
  else if (val == 0x012) sprintf(flagName,FLAG_012);
  else if (val == 0x013) sprintf(flagName,FLAG_013);
  else if (val == 0x014) sprintf(flagName,FLAG_014);
  else if (val == 0x015) sprintf(flagName,FLAG_015);
  else if (val == 0x016) sprintf(flagName,FLAG_016);
  else if (val == 0x017) sprintf(flagName,FLAG_017);
  else if (val == 0x018) sprintf(flagName,FLAG_018);
  else if (val == 0x019) sprintf(flagName,FLAG_019);
  else if (val == 0x01A) sprintf(flagName,FLAG_01A);
  else if (val == 0x01B) sprintf(flagName,FLAG_01B);
  else if (val == 0x01C) sprintf(flagName,FLAG_01C);
  else if (val == 0x01D) sprintf(flagName,FLAG_01D);
  else if (val == 0x01E) sprintf(flagName,FLAG_01E);
  else if (val == 0x01F) sprintf(flagName,FLAG_01F);

  else sprintf(flagName,FLAG_FFF);

  return flagName;
}

/* *** Converts an hexa digit to decimal *** */

unsigned int hex_to_dec(char ch) {
  if (ch >= '0' && ch <= '9') {
    return ((unsigned int) (ch - '0'));
  } else if (ch >= 'a' && ch <= 'f') {
    return 10 + ((unsigned int) (ch - 'a'));
  } else if (ch >= 'A' && ch <= 'F') {
    return 10 + ((unsigned int) (ch - 'A'));
  }

  return 0;
}

/* *** Searchs, reads and returns next variable in buffer *** */

char *scene_import_get_short_hex (char *buffer, int *count, unsigned short int *var) {
  char *beginning;
  char *end;

  /* Looking for variable beginning */
  beginning = strstr(buffer,"[");
  if (beginning == NULL) {
    printf("scene_import_get_var_short_hex : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer; 
  }

  /* Looking for variable end */
  end = strstr(beginning,"]");
  if (end == NULL) {
    printf("scene_import_get_var_short_hex : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer;
  }

  /* If there isn't any value between brackets */
  if (end == beginning + 1) {
    *count = 0;
    return buffer; 
  }

  while (*beginning != 'x' && *(beginning-1) != '0' && *beginning != ']') {
    beginning++;
  }
  if (*beginning == ']') {
    printf("scene_import_get_var_short_hex : ");
    printf(IMPORT_WARNING);
    return 0;
  }
  beginning++;

  *var = (unsigned short int) (hex_to_dec(*(beginning)) * 256 * 16 + hex_to_dec(*(beginning+1)) * 256 +
			       hex_to_dec(*(beginning+2)) * 16 + hex_to_dec(*(beginning+3)) );
  *count = 1;
  return end;
}

char *scene_import_get_short (char *buffer, int *count, short int *var) {
  char *beginning;
  char *end;

  /* Looking for variable beginning */
  beginning = strstr(buffer,"[");
  if (beginning == NULL) {
    printf("scene_import_get_var_short : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer; 
  }

  /* Looking for variable end */
  end = strstr(beginning,"]");
  if (end == NULL) {
    printf("scene_import_get_var_short : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer;
  }

  /* If there isn't any value between brackets */
  if (end == beginning + 1) {
    *count = 0;
    return buffer; 
  }

  while (*beginning != '-' && (*beginning < '0' || *beginning > '9') && *beginning != ']') {
    beginning++;
  }
  if (*beginning == ']') {
    printf("scene_import_get_var_short : ");
    printf(IMPORT_WARNING);
    return 0;
  }

  *var = (short int) get_value(beginning);

  *count = 1;
  return end;
}

char *scene_import_get_uchar_hex (char *buffer, int *count, unsigned char *var) {
  char *beginning;
  char *end;
  int i;

  /* Looking for variable beginning */
  beginning = strstr(buffer,"[");
  if (beginning == NULL) {
    printf("scene_import_get_var_uchar_hex : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer; 
  }

  /* Looking for variable end */
  end = strstr(beginning,"]");
  if (end == NULL) {
    printf("scene_import_get_var_uchar_hex : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer;
  }

  /* If there isn't any value between brackets */
  if (end == beginning + 1) {
    *count = 0;
    return buffer; 
  }

  while (*beginning != 'x' && *(beginning-1) != '0' && *beginning != ']') {
    beginning++;
  }
  if (*beginning == ']') {
    printf("scene_import_get_var_uchar_hex : ");
    printf(IMPORT_WARNING);
    return 0;
  }
  beginning++;

  i = 16 * hex_to_dec(*beginning) + hex_to_dec(*(beginning+1));
  *var = i;
  *count = 1;
  return end;
}

char *scene_import_get_uchar (char *buffer, int *count, unsigned char *var) {
  char *beginning;
  char *end;

  /* Looking for variable beginning */
  beginning = strstr(buffer,"[");
  if (beginning == NULL) {
    printf("scene_import_get_var_uchar : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer; 
  }

  /* Looking for variable end */
  end = strstr(beginning,"]");
  if (end == NULL) {
    printf("scene_import_get_var_uchar : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer;
  }

  /* If there isn't any value between brackets */
  if (end == beginning + 1) {
    *count = 0;
    return buffer; 
  }

  while ((*beginning < '0' || *beginning > '9') && *beginning != ']') {
    beginning++;
  }
  if (*beginning == ']') {
    printf("scene_import_get_var_uchar : ");
    printf(IMPORT_WARNING);
    return 0;
  }

  *var = (unsigned char) get_value(beginning);

  *count = 1;
  return end;
}

char *scene_import_get_int_hex (char *buffer, int *count, unsigned int *var) {
  char *beginning;
  char *end;

  /* Looking for variable beginning */
  beginning = strstr(buffer,"[");
  if (beginning == NULL) {
    printf("scene_import_get_var_int_hex : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer; 
  }

  /* Looking for variable end */
  end = strstr(beginning,"]");
  if (end == NULL) {
    printf("scene_import_get_var_int_hex : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer;
  }

  /* If there isn't any value between brackets */
  if (end == beginning + 1) {
    *count = 0;
    return buffer; 
  }

  while (*beginning != 'x' && *(beginning-1) != '0' && *beginning != ']') {
    beginning++;
  }
  if (*beginning == ']') {
    printf("scene_import_get_var_int_hex : ");
    printf(IMPORT_WARNING);
    return 0;
  }
  beginning++;

  *var = (hex_to_dec(*(beginning+7))               + hex_to_dec(*(beginning+6)) * 16               +
	  hex_to_dec(*(beginning+5)) * 256         + hex_to_dec(*(beginning+4)) * 256 * 16         +
	  hex_to_dec(*(beginning+3)) * 65536       + hex_to_dec(*(beginning+2)) * 65536 * 16       +
	  hex_to_dec(*(beginning+1)) * 65536 * 256 + hex_to_dec(*(beginning  )) * 65536 * 256 * 16);

  *count = 1;
  return end;
}

char *scene_import_get_int (char *buffer, int *count, int *var) {
  char *beginning;
  char *end;

  /* Looking for variable beginning */
  beginning = strstr(buffer,"[");
  if (beginning == NULL) {
    printf("scene_import_get_var_int : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer; 
  }

  /* Looking for variable end */
  end = strstr(beginning,"]");
  if (end == NULL) {
    printf("scene_import_get_var_int : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer;
  }

  /* If there isn't any value between brackets */
  if (end == beginning + 1) {
    *count = 0;
    return buffer; 
  }

  while (*beginning != '-' && (*beginning < '0' || *beginning > '9') && *beginning != ']') {
    beginning++;
  }
  if (*beginning == ']') {
    printf("scene_import_get_var_int : ");
    printf(IMPORT_WARNING);
    return 0;
  }

  *var = get_value(beginning);
  *count = 1;
  return end;
}

char *scene_import_get_text (char *buffer, int *count, char *var) {
  char *beginning;
  char *end;
  int i,brk;

  /* Looking for variable beginning */
  beginning = strstr(buffer,"[");
  if (beginning == NULL) {
    printf("scene_import_get_var_text : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer; 
  }

  /* Looking for variable end */
  end = strstr(beginning,"]");
  if (end == NULL) {
    printf("scene_import_get_var_text : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer;
  }

  /* If there isn't any value between brackets */
  if (end == beginning + 1) {
    *count = 0;
    return buffer; 
  }

  beginning++;
  text_restore((unsigned char *) beginning,*count);
  memcpy(var,beginning,*count);
  text_format((unsigned char *) beginning,*count);

  /* Fixing string returned from text restore */
  brk=0;
  i=(*count) - 1;
  while (i>=0 && brk == 0) {
    if (((unsigned char) var[i]) == 0x00) {
      var[i] = (char) 0xFF;
    } else {
      brk=1;
    }
    i--;
  } 

  return end;
}

char *scene_import_get_char_hex_ptr(char *buffer, int *count, unsigned char **var) {
  char *beginning;
  char *end;
  char *bufferptr;
  int i=0;
  int nbElements;
  unsigned char *retBuffer=NULL;

  /* Looking for variable beginning */
  beginning = strstr(buffer,"[");
  if (beginning == NULL) {
    printf("scene_import_get_char_hex_ptr : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer; 
  }

  /* Looking for variable end */
  end = strstr(beginning,"]");
  if (end == NULL) {
    printf("scene_import_get_char_hex_ptr : ");
    printf(IMPORT_WARNING);
    *count = 0;
    return buffer;
  }

  /* If there isn't any value between brackets */
  if (end == beginning + 1) {
    *count = 0;
    *var=NULL;
    return end; 
  }

  beginning++;
  bufferptr=beginning;
  nbElements=0;

  /* Counting number of bytes in expression */
  while (*bufferptr != ']') {
    if (( (*bufferptr >= '0' && *bufferptr <= '9') ||
	  (*bufferptr >= 'a' && *bufferptr <= 'f') ||
	  (*bufferptr >= 'A' && *bufferptr <= 'F') ) &&
	( (*(bufferptr+1) >= '0' && *(bufferptr+1) <= '9') ||
	  (*(bufferptr+1) >= 'a' && *(bufferptr+1) <= 'f') ||
	  (*(bufferptr+1) >= 'A' && *(bufferptr+1) <= 'F') )) {
      
      nbElements++;
      bufferptr+=2;
      
    } else if (*bufferptr == '\"') {
      bufferptr++;
      while (*bufferptr != '\"') {
	nbElements++;
	bufferptr++;
      }
      nbElements++; /* Last double quote will be 0xFF byte */
      bufferptr++;
    } else {
      bufferptr++;
    }
  } /* end while (*bufferptr != ']') */
  
  *count = nbElements;

  /* Reading data */
  retBuffer = (unsigned char *) malloc(nbElements*sizeof(unsigned char));
  bufferptr=beginning;
  i=0;

  while (*bufferptr != ']') {
    if (( (*bufferptr >= '0' && *bufferptr <= '9') ||
	  (*bufferptr >= 'a' && *bufferptr <= 'f') ||
	  (*bufferptr >= 'A' && *bufferptr <= 'F') ) &&
	( (*(bufferptr+1) >= '0' && *(bufferptr+1) <= '9') ||
	  (*(bufferptr+1) >= 'a' && *(bufferptr+1) <= 'f') ||
	  (*(bufferptr+1) >= 'A' && *(bufferptr+1) <= 'F') )) {
    
      retBuffer[i] = ((unsigned char) (hex_to_dec(*bufferptr) * 16 + hex_to_dec(*(bufferptr+1))));
      bufferptr+=2;
      i++;

    } else if (*bufferptr == '\"') {
      bufferptr++;
      while (*bufferptr != '\"') {
	text_restore((unsigned char *) bufferptr,1);
	retBuffer[i] = (unsigned char) *bufferptr;
	i++;
	bufferptr++;
      }
      retBuffer[i] = 0xFF; /* Last double quote will is a 0xFF byte */
      i++;
      bufferptr++;
      
    } else {
      bufferptr++;
    }
  } /* end while (*bufferptr != ']') */
  
  *var = retBuffer;
 
  return end;   
}

/* *** Imports a formatted text file *** */

scene_t *scene_import_text(char *fileName) {
  scene_t *scene;
  int i,j,k,filenum,count;
  int fileSize;
  FILE *fptr;
  char *buffer,*bufferptr;
  unsigned char *buffertemp;

  /* Looking for text file */
  fptr = fopen(fileName, "rb");
  if (fptr == NULL) {
    printf("scene_import_text : ");
    printf(ERROR_OPEN_FILE, fileName);
    return NULL;
  }

  /* Loading file into memory */
  fileSize = file_length(fptr);
  buffer = (char *) malloc(fileSize*sizeof(char));
  fread(buffer,sizeof(char),fileSize,fptr);
  fclose(fptr);
  bufferptr=buffer;
  
  /* Allocating scene structure */
  scene = (scene_t *) malloc(sizeof(scene_t));

  /* Main loop */
  for (filenum=0;filenum<256;filenum++) {

    /* File structure allocation */
    scene->file[filenum] = (file_type_t *) malloc(sizeof(file_type_t));

    /* Initializing all scripts */
    for (i=0;i<3;i++) {
      for (j=0;j<16;j++) {
	scene->file[filenum]->scripts.block[i].subBlock[j] = NULL;
	scene->file[filenum]->scripts.block[i].subBlockSize[j] = 0;
	scene->file[filenum]->scripts.block[i].subPointer[j] = -1;
      }
    }

    /* Monster ID */
    for (i=0;i<4;i++) {
      bufferptr = scene_import_get_short_hex(bufferptr,&count,&(scene->file[filenum]->monsterID[i]));
    }

    /* Battle global information */
    for (i=0;i<4;i++) {
      bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->battle[i].placeID));
    }
    for (j=0;j<3;j++) {
      for (i=0;i<4;i++) {
	bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->battle[i].unknown[j]));
      }
    }
    for (j=0;j<4;j++) {
      for (i=0;i<4;i++) {
	bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->battle[i].unknown2[j]));
      }
    }
    for (i=0;i<4;i++) {
      bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->battle[i].endBattleID));
    }
    for (i=0;i<4;i++) {
      bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->battle[i].begBattleID));
    }

    /* Battle camera ID */
    for (j=0;j<4;j++) {
      for (i=0;i<4;i++) {
	for (k=0;k<6;k++) {
	  bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->battle_cam[j].cameraData[i][k]));
	}
      }
    }

    /* Battle scene information */
    for (i=0;i<4;i++) {
      for (j=0;j<6;j++) {
	bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->battle_spec[i][j].monsterID));
	bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->battle_spec[i][j].axisX));
	bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->battle_spec[i][j].axisY));
	bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->battle_spec[i][j].axisZ));
	bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->battle_spec[i][j].line));
	bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->battle_spec[i][j].line2));
	bufferptr = scene_import_get_int      (bufferptr,&count,(int *) &(scene->file[filenum]->battle_spec[i][j].undefined));
      }
    }

    /* Monster specifications */
    for (i=0;i<3;i++) {
      count = 32;
      bufferptr = scene_import_get_text (bufferptr,&count,(char *) (scene->file[filenum]->monsterSpec[i].name));
      bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.level);
      bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.speed);
      bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.luck);
      bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.physic_esq);
      bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.strength);
      bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.physic_def);
      bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.magi);
      bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.magic_def);

      for (j=0;j<8;j++) {
	bufferptr = scene_import_get_uchar_hex(bufferptr,&count,(unsigned char *) &scene->file[filenum]->monsterSpec[i].elements[j]);
	bufferptr = scene_import_get_uchar_hex(bufferptr,&count,(unsigned char *) &scene->file[filenum]->monsterSpec[i].links[j]);
      }
      for (j=0;j<16;j++) {
	bufferptr = scene_import_get_uchar_hex(bufferptr,&count,(unsigned char *) &scene->file[filenum]->monsterSpec[i].actionID[j]);
	bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].actionTextID[j]));
	bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].cameraID[j]));
      }
      for (j=0;j<4;j++) {
	bufferptr = scene_import_get_uchar_hex(bufferptr,&count,(unsigned char *) &scene->file[filenum]->monsterSpec[i].link_object[j]);
	bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].object[j]));
      }
      for (j=0;j<3;j++) {
	bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].manipAttackID[j]));
      }
      bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].attackProbability));
      bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->monsterSpec[i].MP));
      bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->monsterSpec[i].AP));
      bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].trans_object));
      for (j=0;j<2;j++) {
	bufferptr = scene_import_get_uchar_hex(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].unknown2[j]);
      }
      bufferptr = scene_import_get_int(bufferptr,&count,(int *) &(scene->file[filenum]->monsterSpec[i].HP));
      bufferptr = scene_import_get_int(bufferptr,&count,(int *) &(scene->file[filenum]->monsterSpec[i].EXP));
      bufferptr = scene_import_get_int(bufferptr,&count,(int *) &(scene->file[filenum]->monsterSpec[i].gils));
      for (j=0;j<4;j++) {
	scene->file[filenum]->monsterSpec[i].flags[j] = 0;
      }
      for (j=0;j<32;j++) {
	bufferptr = scene_import_get_int(bufferptr,&count,(int *) &k);
	if (k == 1) {
	  scene->file[filenum]->monsterSpec[i].flags[j/8] = scene->file[filenum]->monsterSpec[i].flags[j/8] +
	    ((unsigned char) (1 << (7-(j%8)) ));
	}
      }
      for (j=0;j<4;j++) {
	bufferptr = scene_import_get_uchar_hex(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].unknown4[j]);
      }
    }

    /* Attack Data */
    for (k=0;k<32;k++) {
      bufferptr = scene_import_get_char_hex_ptr(bufferptr,&count,&buffertemp);
      if (count != 28) {
	printf("scene_import_text : ");
	printf(IMPORT_ERROR,filenum,"Attack Data");
	return NULL;
      }
      memcpy(scene->file[filenum]->attacks.unknownAtt[k],buffertemp,28);
      if (buffertemp != NULL) {
	free(buffertemp);
	buffertemp=NULL;
      }
    }
    for (k=0;k<32;k++) {
      bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->attacks.nameID[k]));
      count=32;
      bufferptr = scene_import_get_text(bufferptr,&count,(char *) &(scene->file[filenum]->attacks.attackName[k]));
    }
    for (k=0;k<16;k++) {
      bufferptr = scene_import_get_char_hex_ptr(bufferptr,&count,&buffertemp);
      if (count != 16) {
	printf("scene_import_text : ");
	printf(IMPORT_ERROR,filenum,"Attack Data - Last part");
	return NULL;
      }
      memcpy(&scene->file[filenum]->attacks.unknown[k*32],buffertemp,16);
      if (buffertemp != NULL) {
	free(buffertemp);
	buffertemp=NULL;
      }

      bufferptr = scene_import_get_char_hex_ptr(bufferptr,&count,&buffertemp);
      if (count != 16) {
	printf("scene_import_text : ");
	printf(IMPORT_ERROR,filenum,"Attack Data - Last part");
	return NULL;
      }
      memcpy(&scene->file[filenum]->attacks.unknown[k*32+16],buffertemp,16);
      if (buffertemp != NULL) {
	free(buffertemp);
	buffertemp=NULL;
      }
    }

    /* Scripts */
    buffertemp=NULL;
    for (i=0;i<3;i++) {
      for (j=0;j<16;j++) {
	bufferptr = scene_import_get_char_hex_ptr(bufferptr,&k,&scene->file[filenum]->scripts.block[i].subBlock[j]);
	scene->file[filenum]->scripts.block[i].subBlockSize[j] = (short int) k;
      } /* End of for (j) */
    } /* End of for (i) */


  } /* end for filenum */

  free(buffer);

  return scene;
}

/* *** Returns a numeric version of value pointed by ch *** */

int get_value(char *ch) {
  int i=0;
  int sg=1;
  char *charptr;

  charptr=ch;
  /* If value is less than 0 */
  if (ch[0] == '-') {
    sg=-1;
    charptr++;
  }

  /* Number conversion */
  while (*charptr >= '0' && *charptr <= '9') {
    i=i*10;
    i+=((int) (*charptr - '0'));
    charptr++;
  }
 
  i*=sg;

  return i;
}

/* *** Exports monster specifications (only) from scene structure to a text file *** */

int scene_monsters_export_text(scene_t *scene, char *fileName) {
  int filenum,i,j,k;
  int ID,found;
  FILE *fptr;
  char *buffer;

  /* Verification of scene validity */
  if (scene == NULL) {
    printf("scene_monsters_export_text : ");
    printf(SCENE_NULL);
    return 1;
  }
  for (i=0;i<256;i++) {
    if (scene->file[i] == NULL) {
      printf("scene_monsters_export_text : ");
      printf(SCENE_FILE_NULL,i);
      return 2;
    }
  }
  
  /* Trying to create output file */
  fptr = fopen(fileName,"rb");
  if (fptr != NULL) {
    printf("scene_monsters_export_text : ");
    printf(FILE_OVERWRITE,fileName);
    fclose(fptr);
  }
  fptr = fopen(fileName,"wb");
  if (fptr == NULL) {
    printf("scene_monsters_export_text : ");
    printf(ERROR_CREA_FILE,fileName);
    return 3;
  }

  fprintf(fptr,EXPORT_HEADER);
  fprintf(fptr,EXPORT_WARNING);

  for (ID=0;ID<32767;ID++) {
    found=0;
    for (filenum=0;filenum<256 && found == 0;filenum++) {
      for (i=0;i<3 && found == 0;i++) {
	if (scene->file[filenum]->monsterID[i] == ID) {
	  found = 1;
	  
	  /* Exporting monster specifications */
	  fprintf(fptr,"*-------------------------------------------------------------------------------*\n");
	  fprintf(fptr,"| ======================== MONSTER[0x%04x]SPECIFICATIONS ====================== |\n",ID);
	  fprintf(fptr,"*-------------------------------------------------------------------------------*\n");
	  text_format(scene->file[filenum]->monsterSpec[i].name,32);     
	  fprintf(fptr,"| ID 0x%04x name=[%s",
		  scene->file[filenum]->monsterID[i],
		  scene->file[filenum]->monsterSpec[i].name);
	  for(k=0;k<32-strlen((char *) scene->file[filenum]->monsterSpec[i].name);k++) { fprintf(fptr," "); }
	  fprintf(fptr,"]                             |\n");
	  fprintf(fptr,"|                                                                               |\n");
	  text_restore(scene->file[filenum]->monsterSpec[i].name,32);     
	  fprintf(fptr,"| Lvl [%03d] Spd [%03d] Lck [%03d] Esq [%03d] Str [%03d] Df [%03d] Mg [%03d] DefM [%03d]|\n",
		  scene->file[filenum]->monsterSpec[i].main_carac.level,
		  scene->file[filenum]->monsterSpec[i].main_carac.speed,
		  scene->file[filenum]->monsterSpec[i].main_carac.luck,
		  scene->file[filenum]->monsterSpec[i].main_carac.physic_esq,
		  scene->file[filenum]->monsterSpec[i].main_carac.strength,
		  scene->file[filenum]->monsterSpec[i].main_carac.physic_def,
		  scene->file[filenum]->monsterSpec[i].main_carac.magi,
		  scene->file[filenum]->monsterSpec[i].main_carac.magic_def);
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"|              Element                  |   Link between monster and element    |\n");
	  for (j=0;j<8;j++) {
	    buffer = element_name(scene->file[filenum]->monsterSpec[i].elements[j]);
	    fprintf(fptr,"| [0x%02x] %s  ",
		    (unsigned char) scene->file[filenum]->monsterSpec[i].elements[j],
		    buffer);
	    for(k=0;k<29-strlen(buffer);k++) { fprintf(fptr," "); }
	    free(buffer);

	    buffer = link_name(scene->file[filenum]->monsterSpec[i].links[j]);
	    fprintf(fptr,"| [0x%02x] %s  ",
		    (unsigned char) scene->file[filenum]->monsterSpec[i].links[j],
		    buffer);
	    for(k=0;k<29-strlen(buffer);k++) { fprintf(fptr," "); }
	    fprintf(fptr,"|\n");
	    free(buffer);
	  }
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"| attack ID                | attack text ID           | camera ID               |\n");
	  for (j=0;j<16;j++) {
	    fprintf(fptr,"| [0x%02x]                   | [0x%04x]                 | [0x%04x]                |\n",
		    (unsigned char) scene->file[filenum]->monsterSpec[i].actionID[j],
		    (unsigned short int) scene->file[filenum]->monsterSpec[i].actionTextID[j],
		    (unsigned short int) scene->file[filenum]->monsterSpec[i].cameraID[j]);
	  }
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"| Method and probability to obtain it |                Object ID                |\n");
	  for (j=0;j<4;j++) {
	    buffer = object_name(scene->file[filenum]->monsterSpec[i].object[j]);
	    fprintf(fptr,"| [0x%02x]                              | [0x%04x] %s",
		    (unsigned char) scene->file[filenum]->monsterSpec[i].link_object[j],
		    (unsigned short int) scene->file[filenum]->monsterSpec[i].object[j],
		    buffer);
	    for(k=0;k<31-strlen(buffer);k++) { fprintf(fptr," "); }
	    fprintf(fptr,"|\n");
	    free(buffer);
	  }
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"| IDs of attacks player can do when manipulated : [0x%04x] [0x%04x] [0x%04x]    |\n",
		  (unsigned short int) scene->file[filenum]->monsterSpec[i].manipAttackID[0],
		  (unsigned short int) scene->file[filenum]->monsterSpec[i].manipAttackID[1],
		  (unsigned short int) scene->file[filenum]->monsterSpec[i].manipAttackID[2]);
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"| I don t remind why, but we called this param. attackProbability (?) [0x%04x]  |\n",
		  (unsigned short int) scene->file[filenum]->monsterSpec[i].attackProbability);
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"| MP [%05d] AP [%05d]      ID of object obtained with Trans. materia [0x%04x] |\n",
		  scene->file[filenum]->monsterSpec[i].MP,
		  scene->file[filenum]->monsterSpec[i].AP,
		  (unsigned short int) scene->file[filenum]->monsterSpec[i].trans_object);
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"| unknown [0x%02x] [0x%02x]   HP [%010d]  EXP[%010d]  gils [%010d]   |\n",
		  scene->file[filenum]->monsterSpec[i].unknown2[0],
		  scene->file[filenum]->monsterSpec[i].unknown2[1],
		  scene->file[filenum]->monsterSpec[i].HP,
		  scene->file[filenum]->monsterSpec[i].EXP,
		  scene->file[filenum]->monsterSpec[i].gils);
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"|                              ---- FLAGS ----                                  |\n");
	  for (j=0;j<32;j++) {
	    buffer = flag_name(j);
	    fprintf(fptr,"| [%d] %s",
		    ((scene->file[filenum]->monsterSpec[i].flags[j/8] & ((unsigned char) (1 << (7-(j%8)) ))) == 0)?0:1,
		    buffer);
	    for(k=0;k<33-strlen(buffer);k++) { fprintf(fptr," "); }
	    if (j%2 == 1) {
	      fprintf(fptr," |\n");
	    } else {
	      fprintf(fptr," ");
	    }
	    free(buffer);
	  }
	  fprintf(fptr,"|                                                                               |\n");
	  fprintf(fptr,"| unknown [0x%02x] [0x%02x] [0x%02x] [0x%02x]                                           |\n",
		  scene->file[filenum]->monsterSpec[i].unknown4[0],
		  scene->file[filenum]->monsterSpec[i].unknown4[1],
		  scene->file[filenum]->monsterSpec[i].unknown4[2],
		  scene->file[filenum]->monsterSpec[i].unknown4[3]);

	}/* end if (scene->file[filenum]->monsterID[i] == ID) */
      } /* for (i=0;i<3 && found == 0;i++) */
    } /* for (filenum=0;filenum<256 && found == 0;filenum++) */
  } /* for (ID=0;ID<32767;ID++) */

  fclose(fptr);

  return 0;
}

/* *** Imports a formatted text file containing monster specifications *** */

scene_t *scene_monsters_import_text(scene_t *scene, char *fileName) {
  int i,j,k,filenum,count;
  int fileSize;
  FILE *fptr;
  char *buffer,*bufferptr,*bufferpos;
  short int ID;

  /* Verification of scene validity */
  if (scene == NULL) {
    printf("scene_monsters_export_text : ");
    printf(SCENE_NULL);
    return scene;
  }
  for (i=0;i<256;i++) {
    if (scene->file[i] == NULL) {
      printf("scene_monsters_export_text : ");
      printf(SCENE_FILE_NULL,i);
      return scene;
    }
  }
  
  /* Looking for text file */
  fptr = fopen(fileName, "rb");
  if (fptr == NULL) {
    printf("scene_import_text : ");
    printf(ERROR_OPEN_FILE, fileName);
    return scene;
  }

  /* Loading file into memory */
  fileSize = file_length(fptr);
  buffer = (char *) malloc(fileSize*sizeof(char));
  fread(buffer,sizeof(char),fileSize,fptr);
  fclose(fptr);
  bufferptr=buffer;
  
  while (strstr(bufferptr,"[") != NULL) {

    /* Monster ID */
    bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &ID);

    bufferpos=bufferptr;
    for (filenum=0;filenum<256;filenum++) {
      for (i=0;i<3;i++) {
	if (scene->file[filenum]->monsterID[i] == ID) {
	  /* Updating monster specifications */
	  bufferptr=bufferpos;

	  count = 32;
	  bufferptr = scene_import_get_text (bufferptr,&count,(char *) (scene->file[filenum]->monsterSpec[i].name));
	  bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.level);
	  bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.speed);
	  bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.luck);
	  bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.physic_esq);
	  bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.strength);
	  bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.physic_def);
	  bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.magi);
	  bufferptr = scene_import_get_uchar(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].main_carac.magic_def);
	  
	  for (j=0;j<8;j++) {
	    bufferptr = scene_import_get_uchar_hex(bufferptr,&count,(unsigned char *) &scene->file[filenum]->monsterSpec[i].elements[j]);
	    bufferptr = scene_import_get_uchar_hex(bufferptr,&count,(unsigned char *) &scene->file[filenum]->monsterSpec[i].links[j]);
	  }
	  for (j=0;j<16;j++) {
	    bufferptr = scene_import_get_uchar_hex(bufferptr,&count,(unsigned char *) &scene->file[filenum]->monsterSpec[i].actionID[j]);
	    bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].actionTextID[j]));
	    bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].cameraID[j]));
	  }
	  for (j=0;j<4;j++) {
	    bufferptr = scene_import_get_uchar_hex(bufferptr,&count,(unsigned char *) &scene->file[filenum]->monsterSpec[i].link_object[j]);
	    bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].object[j]));
	  }
	  for (j=0;j<3;j++) {
	    bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].manipAttackID[j]));
	  }
	  bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].attackProbability));
	  bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->monsterSpec[i].MP));
	  bufferptr = scene_import_get_short    (bufferptr,&count,(short int *) &(scene->file[filenum]->monsterSpec[i].AP));
	  bufferptr = scene_import_get_short_hex(bufferptr,&count,(unsigned short int *) &(scene->file[filenum]->monsterSpec[i].trans_object));
	  for (j=0;j<2;j++) {
	    bufferptr = scene_import_get_uchar_hex(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].unknown2[j]);
	  }
	  bufferptr = scene_import_get_int(bufferptr,&count,(int *) &(scene->file[filenum]->monsterSpec[i].HP));
	  bufferptr = scene_import_get_int(bufferptr,&count,(int *) &(scene->file[filenum]->monsterSpec[i].EXP));
	  bufferptr = scene_import_get_int(bufferptr,&count,(int *) &(scene->file[filenum]->monsterSpec[i].gils));
	  for (j=0;j<4;j++) {
	    scene->file[filenum]->monsterSpec[i].flags[j] = 0;
	  }
	  for (j=0;j<32;j++) {
	    bufferptr = scene_import_get_int(bufferptr,&count,(int *) &k);
	    if (k == 1) {
	      scene->file[filenum]->monsterSpec[i].flags[j/8] = scene->file[filenum]->monsterSpec[i].flags[j/8] +
		((unsigned char) (1 << (7-(j%8)) ));
	    }
	  }
	  for (j=0;j<4;j++) {
	    bufferptr = scene_import_get_uchar_hex(bufferptr,&count,&scene->file[filenum]->monsterSpec[i].unknown4[j]);
	  }
	}
      }
    }
  }

  free(buffer);

  return scene;
}
