#ifndef _scene_
#define _scene_

#include <stdlib.h>
#include <stdio.h>
#include "../common/common.h"
#include <zlib.h>


/* *** Structures definitions
       Total size : 7808 bytes / 0x1E80 bytes
       Each battle scene structure has got the same length
       Each monster structure has got the same length      *** */


typedef struct _battle_scene {
  short int placeID;              /* Grass, Shinra, Rocket city, ... */
  short int unknown[3];           /* Unknown values */
  short int unknown2[4];          /* Often four same values (often 999) */
  short int endBattleID;          /* What is done when battle is finished (fanfare, nothing, ...) */
  short int begBattleID;          /* What is done before beginning of battle */
} battle_scene_t;

typedef struct _battle_scene_camera {
  short int cameraData[4][6];     /* Has something to do with camera coordinates */
} battle_scene_camera_t;

typedef struct _battle_scene_spec {
  short int monsterID;            /* monster ID number (see beginning of filetype) */
  short int axisX;                /* monster X coordinates */
  short int axisY;                /* monster Y coordinates */
  short int axisZ;                /* monster Z coordinates */
  short int line;                 /* "line" where the monster is. (First or second) */
  short int line2;                /* SEEMS to have something to do with line        */
  int undefined;                  /* usually -1 (if -2, monster will be vanished)   */
} battle_scene_spec_t;

typedef struct _monster_main_carac {
  unsigned char level;            /* monster's level */
  unsigned char speed;            /* monster's speed */
  unsigned char luck;             /* luck */
  unsigned char physic_esq;       /* percent of physical dodge */
  unsigned char strength;         /* strength */
  unsigned char physic_def;       /* physical defense */
  unsigned char magi;             /* magic power */
  unsigned char magic_def;        /* magic defense */
} monster_main_carac_t;

typedef struct _monster_carac {
  unsigned char name[32];         /* monster's name */
  monster_main_carac_t main_carac;/* see struct above */
  char elements[8];               /* list of elements */
  char links[8];                  /* "links" between elements and monster (absorb, resist...) */
  char actionID[16];              /* ID of monster special attacks */
  short int actionTextID[16];     /* ID of special attacks names */
  short int cameraID[16];         /* ID of camera movement, used during a special attack */
  char link_object[4];            /* method and probability to obtain object */
  short int object[4];            /* object ID */
  short int manipAttackID[3];     /* IDs of attacks you can do when monster's manipulated */
  short int attackProbability;    /* -1 seems to be high probability and -2 to be less */
  unsigned short int MP;          /* MP */
  unsigned short int AP;          /* AP when battle's over */
  short int trans_object;         /* object obtained with Trans. materia */
  unsigned char unknown2[2];      /* usually 2 numbers : 10 FF */
  unsigned int HP;                /* HP */
  unsigned int EXP;               /* Exp when battle's over */
  unsigned int gils;              /* gils when battle's over */
  unsigned char flags[4];         /* 32 flags */
  unsigned char unknown4[4];      /* usually FF FF FF FF */
} monster_carac_t;

typedef struct _attacks_names {
  unsigned char unknownAtt[32][28];     /* Unknown values (Has something to do with attacks) */
  short int nameID[32];                 /* ID of special attacks' names (see. monster_carac) */
  unsigned char attackName[32][32];     /* Text which corresponds to special attack */
  unsigned char unknown[512];           /* Has it anything to do with attacks names ? */
} attacks_names_t;

typedef struct _block {
  short int subPointer[16];             /* Points to each sub-block */
  unsigned char *subBlock[16];          /* Up to 16 sub-block */
  short int subBlockSize[16];           /* Size of subBlock */
} block_t;

typedef struct _scripts {
  short int pointer[3];                 /* Up to 3 script blocks (must be even) */
  block_t block[3];                     /* Contains script blocks */
} scripts_t;

typedef struct _file_type {
  unsigned short int monsterID[4];      /* 0x000 IDs of the monsters (up to 3) */
  battle_scene_t battle[4];             /* 0x008 Battle global information */
  battle_scene_camera_t battle_cam[4];  /* 0x058 Has something to do with camera coordinates */
  battle_scene_spec_t battle_spec[4][6];/* 0x118 Battle scenes specifications */
  monster_carac_t monsterSpec[3];       /* 0x298 Description of the 1, 2 or 3 monsters */
  attacks_names_t attacks;              /* 0x4C0 Attack data */
  scripts_t scripts;                    /* 0xE80 Scripts (variable length) */
} file_type_t;

typedef struct _scene_t {
  file_type_t *file[256];               /* Regroups all data from a scene.bin file */
} scene_t;


/* *** Functions prototypes *** */


/* Loads a scene.bin file into a scene structure */
scene_t *scene_load(char *sceneFileName);

/* Saves a scene structure in the specified file (allocate blockTable[64] before call) */
int scene_save(char *sceneFileName, scene_t *scene, unsigned char *blockTable); 

/* Loads an uncompressed "file" (i.e. a four scenes block) */
file_type_t *file_load(char *fileName);   

/* Saves an uncompressed "file" */
void file_save(char *fileName, file_type_t *file); 

/* Erases a scene structure from memory */
void scene_clear(scene_t *scene);   

/* Exports a whole scene structure to a text file */
int scene_export_text(scene_t *scene, char *fileName);

/* Imports a formatted text file */
scene_t *scene_import_text(char *fileName);

/* Exports monster specifications (only) from scene structure to a text file */
int scene_monsters_export_text(scene_t *scene, char *fileName);

/* Imports a formatted text file containing monster specifications */
scene_t *scene_monsters_import_text(scene_t *scene, char *fileName);

/* Converts a string to a readable format */
void text_format(unsigned char *str, int length);

/* Converts a string to the "FF7 format" */
void text_restore(unsigned char *str, int length);

/* Returns an object name from its ID number */
char *object_name(int val);

/* Returns an element name from its ID number */
char *element_name(int val);

/* Returns a link name from its ID number */
char *link_name(int val);

/* Returns name of a flag */
char *flag_name(int val);

/* Converts an hexa digit to decimal */
unsigned int hex_to_dec(char ch);

/* Returns a numeric version of value pointed by ch */
int get_value(char *ch);

/* Searchs, reads and returns next requested variable in buffer (used by scene_import) */
char *scene_import_get_short_hex (char *buffer, int *count, unsigned short int *var);
char *scene_import_get_short (char *buffer, int *count, short int *var);
char *scene_import_get_uchar_hex (char *buffer, int *count, unsigned char *var);
char *scene_import_get_uchar (char *buffer, int *count, unsigned char *var);
char *scene_import_get_int_hex (char *buffer, int *count, unsigned int *var);
char *scene_import_get_int (char *buffer, int *count, int *var);
char *scene_import_get_text (char *buffer, int *count, char *var);
char *scene_import_get_char_hex_ptr(char *buffer, int *count, unsigned char **var);

#endif
