#include "ff7scenetools.h"

int main(int argc, char *argv[]) {

  scene_t *scene;
  kernel_t *kernel;
  int i;
  unsigned char table[64];
  int command;

  printf("\nFF7SceneTools 1.0 - Fremen / Shit Fliez - 2004\n\n");

  /* Reading parameters */
  if (argc < 2) {
    command = OPT_HELP;

  } else if (strcmp(argv[1],"--help") == 0 ||
	     strcmp(argv[1],"-h")     == 0 ||
	     strcmp(argv[1],"/h")     == 0 ||
	     strcmp(argv[1],"/help")  == 0 ||
	     strcmp(argv[1],"/?")     == 0) {
    command = OPT_HELP;

  } else if (strcmp(argv[1],"nrg2iso") == 0 && argc == 4) {
    command = OPT_NRG2ISO;

  } else if (strcmp(argv[1],"bin2iso") == 0 && argc == 4) {
    command = OPT_BIN2ISO;

  } else if (strcmp(argv[1],"iso2bin") == 0 && argc == 4) {
    command = OPT_ISO2BIN;

  } else if (strcmp(argv[1],"kernel-patch") == 0 && argc == 5) {
    command = OPT_KERNEL_P;

  } else if (strcmp(argv[1],"scene-patch") == 0 && argc == 5) {
    command = OPT_SCENE_P;

  } else if (strcmp(argv[1],"scene-export") == 0 && argc == 4) {
    command = OPT_SCENE_EXP;

  } else if (strcmp(argv[1],"scene-import") == 0 && (argc == 4 || argc == 6)) {
    command = OPT_SCENE_IMP;

  } else if (strcmp(argv[1],"monst-export") == 0 && argc == 4) {
    command = OPT_MONST_EXP;

  } else if (strcmp(argv[1],"monst-import") == 0 && (argc == 5 || argc == 7)) {
    command = OPT_MONST_IMP;

  } else {
    printf(UNKNOWN_COMMAND,argv[1]);
    command = OPT_HELP;
  }

  /* Displaying help if asked and/or needed */
  if (command == OPT_HELP) {
    if (argc <= 2) {
      printf(HELP_USAGE,argv[0]);
      printf(HELP_USAGE_2,argv[0]);
      printf(HELP_COMMANDS);
      printf(HELP_COMMAND_1);
      printf(HELP_COMMAND_2);
      printf(HELP_COMMAND_3);
      printf(HELP_COMMAND_4);
      printf(HELP_COMMAND_5);
      printf(HELP_COMMAND_6);
      printf(HELP_COMMAND_7);
      printf(HELP_COMMAND_8);
      printf(HELP_COMMAND_9);
      printf("\n\n");

    } else if (strcmp(argv[2],"nrg2iso") == 0) {
      printf(HELP_COMMAND_1_H1,argv[0]);
      printf(HELP_COMMAND_1_H2);

    } else if (strcmp(argv[2],"bin2iso") == 0) {
      printf(HELP_COMMAND_2_H1,argv[0]);
      printf(HELP_COMMAND_2_H2);

    } else if (strcmp(argv[2],"iso2bin") == 0) {
      printf(HELP_COMMAND_3_H1,argv[0]);
      printf(HELP_COMMAND_3_H2);

    } else if (strcmp(argv[2],"kernel-patch") == 0) {
      printf(HELP_COMMAND_4_H1,argv[0]);
      printf(HELP_COMMAND_4_H2);

    } else if (strcmp(argv[2],"scene-patch") == 0) {
      printf(HELP_COMMAND_5_H1,argv[0]);
      printf(HELP_COMMAND_5_H2);

    } else if (strcmp(argv[2],"scene-export") == 0) {
      printf(HELP_COMMAND_6_H1,argv[0]);
      printf(HELP_COMMAND_6_H2);

    } else if (strcmp(argv[2],"scene-import") == 0) {
      printf(HELP_COMMAND_7_H1,argv[0]);
      printf(HELP_COMMAND_7_H2);

    } else if (strcmp(argv[2],"monst-export") == 0) {
      printf(HELP_COMMAND_8_H1,argv[0]);
      printf(HELP_COMMAND_8_H2);

    } else if (strcmp(argv[2],"monst-import") == 0) {
      printf(HELP_COMMAND_9_H1,argv[0]);
      printf(HELP_COMMAND_9_H2);

    } else {
      printf(UNKNOWN_COMMAND,argv[2]);
      printf(HELP_USAGE,argv[0]);
      printf(HELP_USAGE_2,argv[0]);
      printf(HELP_COMMANDS);
      printf(HELP_COMMAND_1);
      printf(HELP_COMMAND_2);
      printf(HELP_COMMAND_3);
      printf(HELP_COMMAND_4);
      printf(HELP_COMMAND_5);
      printf(HELP_COMMAND_6);
      printf(HELP_COMMAND_7);
      printf(HELP_COMMAND_8);
      printf(HELP_COMMAND_9);
      printf("\n\n");
    }
  }

  /* NRG to ISO convertion */
  if (command == OPT_NRG2ISO) {
    return (nrg2iso(argv[2], argv[3]));
  }

  /* BIN to ISO convertion */
  if (command == OPT_BIN2ISO) {
    return (bin2iso(argv[2], argv[3]));
  }

  /* ISO to BIN convertion */
  if (command == OPT_ISO2BIN) {
    return (iso2bin(argv[2], argv[3]));
  }

  /* Kernel patch command */
  if (command == OPT_KERNEL_P) {
    return (file_patch(argv[2], argv[3], argv[4], ID_KERNEL));
  }

  /* Scene patch command */
  if (command == OPT_SCENE_P) {
    return (file_patch(argv[2], argv[3], argv[4], ID_SCENE));
  }

  /* Scene export command */
  if (command == OPT_SCENE_EXP) {
    scene = scene_load(argv[2]);
    if (scene == NULL) {
      return 1;
    }
    scene_export_text(scene, argv[3]);
    scene_clear(scene);
  }

  /* Scene import command */
  if (command == OPT_SCENE_IMP) {
    if ((scene = scene_import_text(argv[2])) == NULL) {
      return 1;
    }
    if ((i = scene_save(argv[3], scene, table)) != 0) {
      return i;
    }
    scene_clear(scene);

    if (argc == 6) {
      if ((kernel = kernel_load(argv[4])) == NULL) {
	return 2;
      }
      memcpy(&kernel->blockData[2][0xF1C],table,64);
      if ((i == kernel_save(argv[5],kernel)) != 0) {
	return i;
      }
      clear_kernel(kernel);
    }
  }

  /* Monster export command */
  if (command == OPT_MONST_EXP) {
    scene = scene_load(argv[2]);
    if (scene == NULL) {
      return 1;
    }
    if ((i = (scene_monsters_export_text(scene, argv[3]))) != 0) {
      return i;
    }
    scene_clear(scene);
  }

  /* Monster import command */
  if (command == OPT_MONST_IMP) {

    /* Loads original scene */
    scene = scene_load(argv[3]);
    if (scene == NULL) {
      return 1;
    }

    /* Imports bestiary */
    if ((scene = scene_monsters_import_text(scene,argv[2])) == NULL) {
      return 1;
    }

    /* Saving modified scene */
    if ((i = scene_save(argv[4], scene, table)) != 0) {
      return i;
    }
    scene_clear(scene);

    if (argc == 7) {
      if ((kernel = kernel_load(argv[5])) == NULL) {
	return 2;
      }
      memcpy(&kernel->blockData[2][0xF1C],table,64);
      if ((i == kernel_save(argv[6],kernel)) != 0) {
	return i;
      }
      clear_kernel(kernel);
    }
  }

  return 0;
}
